#!/usr/bin/env python

""" MultiQC module to parse output from RNA-SeQC """

from __future__ import print_function
from collections import OrderedDict
import logging

from multiqc import config
from multiqc.modules.base_module import BaseMultiqcModule
from multiqc.plots import bargraph, beeswarm, linegraph, heatmap


# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):
        # Initialise the parent object
        super(MultiqcModule, self).__init__(
            name="RNA-SeQC",
            anchor="rna_seqc",
            href="https://github.com/broadinstitute/rnaseqc",
            info="Fast, efficient RNA-Seq metrics for quality control and process optimization"
        )

        # Parse metrics from RNA-SeQC v1
        self.rna_seqc_metrics = dict()
        for f in self.find_log_files('rna_seqc/metrics_v1', filehandles=True):
            self.parse_metrics_rnaseqc_v1(f)

        # Parse metrics from RNA-SeQC v2
        for f in self.find_log_files('rna_seqc/metrics_v2', filehandles=True):
            self.parse_metrics_rnaseqc_v2(f)

        # Strip out ignored sample names from metrics
        self.rna_seqc_metrics = self.ignore_samples(self.rna_seqc_metrics)

        # Parse normalised coverage information.
        self.rna_seqc_norm_high_cov = dict()
        self.rna_seqc_norm_medium_cov = dict()
        self.rna_seqc_norm_low_cov = dict()
        for f in self.find_log_files('rna_seqc/coverage'):
            self.parse_coverage(f)
        self.rna_seqc_norm_high_cov = self.ignore_samples(self.rna_seqc_norm_high_cov)
        self.rna_seqc_norm_medium_cov = self.ignore_samples(self.rna_seqc_norm_medium_cov)
        self.rna_seqc_norm_low_cov = self.ignore_samples(self.rna_seqc_norm_low_cov)

        # Parse correlation matrices
        self.rna_seqc_pearson = None
        self.rna_seqc_spearman = None
        num_correlation_samples = 0
        for f in self.find_log_files('rna_seqc/correlation'):
            num_correlation_samples += self.parse_correlation(f)

        # Work out the maximum number of samples that we have across all reports
        num_found = max(
            len(self.rna_seqc_metrics),
            len(self.rna_seqc_norm_high_cov),
            len(self.rna_seqc_norm_medium_cov),
            len(self.rna_seqc_norm_low_cov),
            num_correlation_samples
        )

        # Stop here if we didn't find anything
        if num_found == 0:
            raise UserWarning

        log.info("Found {} samples".format(num_found))

        # Write metrics to a file
        self.write_data_file(self.rna_seqc_metrics, 'multiqc_rna_seqc')

        # Generate sections for the report
        self.rnaseqc_general_stats()
        self.transcript_associated_plot()
        self.plot_correlation_heatmap()
        self.strand_barplot()
        self.coverage_lineplot()
        self.bam_statplot()

    def parse_metrics_rnaseqc_v1(self, f):
        """
        Parse the metrics.tsv file from RNA-SeQC version 1.x
        """
        headers = None
        for l in f['f']:
            s = l.strip().split("\t")
            if headers is None:
                headers = s
            else:
                s_name = s[headers.index('Sample')]
                data = dict()
                for idx, h in enumerate(headers):
                    try:
                        data[h] = float(s[idx])
                    except ValueError:
                        data[h] = s[idx]
                if s_name in self.rna_seqc_metrics:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
                self.rna_seqc_metrics[s_name] = data

    def parse_metrics_rnaseqc_v2(self, f):
        """
        Parse the *.metrics.tsv file from RNA-SeQC version 2.x
        """
        data = {}
        s_name = f['s_name']
        for l in f['f']:
            s = l.split('\t')
            if s[0] == 'Sample':
                s_name = self.clean_s_name(s[1], f['root'])
            if s[0] == "Total Reads":
                s[0] = "Total Read Number"
            if s[0] == "rRNA Rate":
                s[0] = "rRNA rate"
            try:
                data[s[0]] = float(s[1])
            except ValueError:
                data[s[0]] = s[1].strip()

        if s_name in self.rna_seqc_metrics:
            log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
        self.rna_seqc_metrics[s_name] = data


    def parse_coverage (self, f):
        """ Parse the RNA-SeQC Normalised Coverage Files """
        data = dict()
        s_names = None
        j = 1
        for l in f['f'].splitlines():
            s = l.strip().split("\t")
            if s_names is None:
                s_names = s
                for s_name in s_names:
                    s_name = self.clean_s_name(s_name, f['root'])
                    data[s_name] = dict()
            else:
                for i, v in enumerate(s):
                    data[s_names[i]][j] = float(v)
                j += 1
        if f['fn'] == 'meanCoverageNorm_high.txt':
            self.rna_seqc_norm_high_cov.update(data)
        elif f['fn'] == 'meanCoverageNorm_medium.txt':
            self.rna_seqc_norm_medium_cov.update(data)
        elif f['fn'] == 'meanCoverageNorm_low.txt':
            self.rna_seqc_norm_low_cov.update(data)

    def parse_correlation(self, f):
        """ Parse RNA-SeQC correlation matrices """
        s_names = None
        data = list()
        for l in f['f'].splitlines():
            s = l.strip().split("\t")
            if s_names is None:
                s_names = [ x for x in s if x != '' ]
            else:
                data.append(s[1:])

        # Filter for ignored sample names
        filtered_s_names = list()
        filtered_data = list()
        for idx, s_name in enumerate(s_names):
            s_name = self.clean_s_name(s_name, f['root'])
            if not self.is_ignore_sample(s_name):
                filtered_s_names.append(s_name)
                filtered_data.append(data[idx])

        if f['fn'] == 'corrMatrixPearson.txt':
            self.rna_seqc_pearson = (filtered_s_names, filtered_data)
        elif f['fn'] == 'corrMatrixSpearman.txt':
            self.rna_seqc_spearman = (filtered_s_names, filtered_data)

        return len(filtered_s_names)


    def rnaseqc_general_stats(self):
        """
        Add key metrics to the general stats table
        """
        headers = OrderedDict()
        headers['Expression Profiling Efficiency'] = {
            'title': '% Expression Efficiency',
            'description': 'Ratio of exonic reads to total reads',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'YlGn',
            'modify': lambda x: float(x) * 100.0
        }
        headers['Genes Detected'] = {
            'title': '# Genes',
            'description': 'Number of genes detected with at least 5 reads',
            'min': 0,
            'scale': 'Bu',
            'format': '{:,.0f}'
        }
        headers['rRNA rate'] = {
            'title': '% rRNA Alignment',
            'description': 'Ribosomal RNA reads per total reads',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'Reds',
            'modify': lambda x: float(x) * 100.0
        }
        self.general_stats_addcols(self.rna_seqc_metrics, headers)

    def transcript_associated_plot (self):
        """ Plot a bargraph showing the Transcript-associated reads  """

        # Plot bar graph of groups
        keys = OrderedDict()
        keys['Exonic Rate'] = { 'name': 'Exonic', 'color': '#2f7ed8' }
        keys['Intronic Rate'] = { 'name': 'Intronic', 'color': '#8bbc21' }
        keys['Intergenic Rate'] = { 'name': 'Intergenic', 'color': '#0d233a'}

        # Config for the plot
        pconfig = {
            'id': 'rna_seqc_position_plot',
            'title': 'RNA-SeQC: Transcript-associated reads',
            'ylab': 'Ratio of Reads',
            'cpswitch': False,
            'ymax': 1,
            'ymin': 0,
            'tt_decimals': 3,
            'cpswitch_c_active': False
        }
        self.add_section (
            name = 'Transcript-associated reads',
            anchor = 'rna_seqc_transcript_associated',
            helptext = '''
            All of the rates given are per mapped read.

            * **Exonic Rate** is the fraction mapping within exons.
            * **Intronic Rate** is the fraction mapping within introns.
            * **Intergenic Rate** is the fraction mapping in the genomic space between genes.
            ''',
            plot = bargraph.plot(self.rna_seqc_metrics, keys, pconfig)
        )

    def plot_correlation_heatmap(self):
        """ Return HTML for correlation heatmap """
        data = None
        corr_type = None
        correlation_type = getattr(config, 'rna_seqc' ,{}).get('default_correlation', 'spearman')
        if self.rna_seqc_spearman is not None and correlation_type != 'pearson':
            data = self.rna_seqc_spearman
            corr_type = 'Spearman'
        elif self.rna_seqc_pearson is not None:
            data = self.rna_seqc_pearson
            corr_type = 'Pearson'
        if data is not None:
            pconfig = {
                'id': 'rna_seqc_correlation_heatmap',
                'title': 'RNA-SeQC: {} Sample Correlation'.format(corr_type)
            }
            self.add_section (
                name = '{} Correlation'.format(corr_type),
                anchor = 'rseqc-rna_seqc_correlation',
                plot = heatmap.plot(data[1], data[0], data[0], pconfig)
            )


    def strand_barplot(self):
        """ Plot a bargraph showing the strandedness of alignments """
        # Plot bar graph of groups
        keys = [ 'End 1 Sense', 'End 1 Antisense', 'End 2 Sense', 'End 2 Antisense' ]
        # Config for the plot
        pconfig = {
            'id': 'rna_seqc_strandedness_plot',
            'title': 'RNA-SeQC: Strand Specificity',
            'ylab': '% Reads',
            'cpswitch_counts_label': '# Reads',
            'cpswitch_percent_label': '% Reads',
            'ymin': 0,
            'cpswitch_c_active': False
        }
        self.add_section (
            name = 'Strand Specificity',
            anchor = 'rna_seqc_strand_specificity',
            helptext = '''
            End 1/2 Sense are the number of End 1 or 2 reads that were sequenced in the sense direction.

            End 1/2 Antisense are the number of End 1 or 2 reads that were sequenced in the antisense direction
            ''',
            plot = bargraph.plot(self.rna_seqc_metrics, keys, pconfig)
        )


    def coverage_lineplot (self):
        """ Make HTML for coverage line plots """
        # Add line graph to section
        data = list()
        data_labels = list()
        if len(self.rna_seqc_norm_high_cov) > 0:
            data.append(self.rna_seqc_norm_high_cov)
            data_labels.append({'name': 'High Expressed'})
        if len(self.rna_seqc_norm_medium_cov) > 0:
            data.append(self.rna_seqc_norm_medium_cov)
            data_labels.append({'name': 'Medium Expressed'})
        if len(self.rna_seqc_norm_low_cov) > 0:
            data.append(self.rna_seqc_norm_low_cov)
            data_labels.append({'name': 'Low Expressed'})
        pconfig = {
            'id': 'rna_seqc_gene_body_coverage_plot',
            'title': 'RNA-SeQC: Gene Body Coverage',
            'ylab': '% Coverage',
            'xlab': "Gene Body Percentile (5' -> 3')",
            'xmin': 0,
            'xmax': 100,
            'tt_label': "<strong>{point.x}% from 5'</strong>: {point.y:.2f}",
            'data_labels': data_labels
        }
        if len(data) > 0:
            self.add_section (
                name = 'Gene Body Coverage',
                anchor = 'rseqc-rna_seqc_mean_coverage',
                helptext = 'The metrics are calculated across the transcripts with tiered expression levels.',
                plot = linegraph.plot(data, pconfig)
            )


    def bam_statplot(self):
        pconfig = {
            'id': 'rna_seqc_bam_stat_beeswarm',
            'title': 'RNA-SeQC: Read metrics'
        }
        columns = [
            'Total Read Number',
            'Alternative Alignments',
            'Chimeric Reads',
            'Duplicate Reads',
            'End 1 Mapped Reads',
            'End 2 Mapped Reads',
            'End 1 Mismatches',
            'End 2 Mismatches',
            'End 1 Sense',
            'End 2 Sense',
            'Ambiguous Reads',
            'High Quality Reads',
            'Low Quality Reads',
            'Mapped Duplicate Reads',
            'Mapped Reads',
            'Mapped Unique Reads',
            'Non-Globin Reads',
            'Non-Globin Duplicate Reads',
            'rRNA Reads',
            'Unique Mapping, Vendor QC Passed Reads'
        ]
        keys = OrderedDict()
        for col in columns:
            keys[col] = {
                'title': col,
                'shared_key': 'read_count',
                'suffix': config.read_count_prefix
            }

        self.add_section(
            name = 'Read Counts',
            anchor = 'rna_seqc_bam_stat',
            description = 'Number of reads ({}) falling into different categories.'.format(config.read_count_desc),
            helptext = 'Note that many of these statistics are only available from RNA-SeQC v2.x',
            plot = beeswarm.plot(self.rna_seqc_metrics, keys, pconfig)
        )



#!/usr/bin/env python

""" MultiQC module to parse output from Peddy """

from __future__ import print_function
from collections import OrderedDict
import logging
import json

from multiqc.plots import scatter
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)


class MultiqcModule(BaseMultiqcModule):
    """
    Peddy module class, parses stderr logs.
    """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='Peddy', anchor='peddy',
        href='https://github.com/brentp/peddy',
        info="calculates genotype :: pedigree correspondence checks, ancestry checks and sex checks using VCF files.")

        # Find and load any Peddy reports
        self.peddy_data = dict()
        self.peddy_length_counts = dict()
        self.peddy_length_exp = dict()
        self.peddy_length_obsexp = dict()

        # parse peddy summary file
        for f in self.find_log_files('peddy/summary_table'):
            parsed_data = self.parse_peddy_summary(f)
            if parsed_data is not None:
                for s_name in parsed_data:
                    s_name = self.clean_s_name(s_name, f['root'])
                    try:
                        self.peddy_data[s_name].update(parsed_data[s_name])
                    except KeyError:
                        self.peddy_data[s_name] = parsed_data[s_name]

        # parse peddy CSV files
        for pattern in ['het_check', 'ped_check', 'sex_check']:
            sp_key = 'peddy/{}'.format(pattern)
            for f in self.find_log_files(sp_key):
                # some columns have the same name in het_check and sex_check (median_depth)
                # pass pattern to parse_peddy_csv so the column names can include pattern to
                # avoid being overwritten
                parsed_data = self.parse_peddy_csv(f, pattern)
                if parsed_data is not None:
                    for s_name in parsed_data:
                        try:
                            self.peddy_data[s_name].update(parsed_data[s_name])
                        except KeyError:
                            self.peddy_data[s_name] = parsed_data[s_name]

        # parse background PCA JSON file, this is identitical for all peddy runs,
        # so just parse the first one we find
        for f in self.find_log_files("peddy/background_pca"):
            background = json.loads(f['f'])
            PC1 = [x["PC1"] for x in background]
            PC2 = [x["PC2"] for x in background]
            ancestry = [x["ancestry"] for x in background]
            self.peddy_data["background_pca"] =  {
                "PC1": PC1, "PC2": PC2, "ancestry": ancestry}
            break

        # Filter to strip out ignored sample names
        self.peddy_data = self.ignore_samples(self.peddy_data)

        if len(self.peddy_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.peddy_data)))

        # Write parsed report data to a file
        self.write_data_file(self.peddy_data, 'multiqc_peddy')

        # Basic Stats Table
        self.peddy_general_stats_table()

        # PCA plot
        self.peddy_pca_plot()

        # Relatedness plot
        self.peddy_relatedness_plot()

        # hetcheck plot
        self.peddy_het_check_plot()

        self.peddy_sex_check_plot()

    def parse_peddy_summary(self, f):
        """ Go through log file looking for peddy output """
        parsed_data = dict()
        headers = None
        for l in f['f'].splitlines():
            s = l.split("\t")
            if headers is None:
                s[0] = s[0].lstrip('#')
                headers = s
            else:
                parsed_data[s[1]] = dict()
                for i, v in enumerate(s):
                    if i != 1:
                        try:
                            parsed_data[s[1]][headers[i]] = float(v)
                        except ValueError:
                            parsed_data[s[1]][headers[i]] = v
        if len(parsed_data) == 0:
            return None
        return parsed_data

    def parse_peddy_csv(self, f, pattern):
        """ Parse csv output from peddy """
        parsed_data = dict()
        headers = None
        s_name_idx = None
        for l in f['f'].splitlines():
            s = l.split(",")
            if headers is None:
                headers = s
                try:
                    s_name_idx = [headers.index("sample_id")]
                except ValueError:
                    try:
                        s_name_idx = [headers.index("sample_a"), headers.index("sample_b")]
                    except ValueError:
                        log.warning("Could not find sample name in Peddy output: {}".format(f['fn']))
                        return None
            else:
                s_name = '-'.join([s[idx] for idx in s_name_idx])
                s_name = self.clean_s_name(s_name, f['root'])
                parsed_data[s_name] = dict()
                for i, v in enumerate(s):
                    if i not in s_name_idx:
                        if headers[i] == "error" and pattern == "sex_check":
                            v = "True" if v == "False" else "False"
                        try:
                            # add the pattern as a suffix to key
                            parsed_data[s_name][headers[i] + "_" + pattern] = float(v)
                        except ValueError:
                            # add the pattern as a suffix to key
                            parsed_data[s_name][headers[i] + "_" + pattern] = v
        if len(parsed_data) == 0:
            return None
        return parsed_data

    def peddy_general_stats_table(self):
        """ Take the parsed stats from the Peddy report and add it to the
        basic stats table at the top of the report """

        family_ids = [ x.get('family_id') for x in self.peddy_data.values() ]

        headers = OrderedDict()
        headers['family_id'] = {
            'title': 'Family ID',
            'hidden': True if all([v == family_ids[0] for v in family_ids]) else False
        }
        headers['ancestry-prediction'] = {
            'title': 'Ancestry',
            'description': 'Ancestry Prediction',
        }
        headers['ancestry-prob_het_check'] = {
            'title': 'P(Ancestry)',
            'description': 'Probability predicted ancestry is correct.'
        }
        headers['sex_het_ratio'] = {
            'title': 'Sex / Het Ratio',
        }
        headers['error_sex_check'] = {
            'title': 'Correct Sex',
            'description': 'Displays False if error in sample sex prediction',
        }
        headers['predicted_sex_sex_check'] = {
            'title': 'Sex',
            'description': 'Predicted sex'
        }
        self.general_stats_addcols(self.peddy_data, headers)

    def peddy_pca_plot(self):
        ancestry_colors = {
            'SAS': 'rgb(68,1,81,1)',
            'EAS': 'rgb(59,81,139,1)',
            'AMR': 'rgb(33,144,141,1)',
            'AFR': 'rgb(92,200,99,1)',
            'EUR': 'rgb(253,231,37,1)'
        }
        background_ancestry_colors = {
            'SAS': 'rgb(68,1,81,0.1)',
            'EAS': 'rgb(59,81,139,0.1)',
            'AMR': 'rgb(33,144,141,0.1)',
            'AFR': 'rgb(92,200,99,0.1)',
            'EUR': 'rgb(253,231,37,0.1)'
        }
        default_color = '#000000'
        default_background_color = 'rgb(211,211,211,0.05)'
        data = OrderedDict()

        # plot the background data first, so it doesn't hide the actual data points
        d = self.peddy_data.pop("background_pca", {})
        if d:
            background = [{'x': pc1,
                        'y': pc2,
                        'color': default_background_color,
                        'name': ancestry,
                        'marker_size': 1}
                        for pc1, pc2, ancestry in zip(d['PC1'], d['PC2'], d['ancestry'])]
            data["background"] = background

        for s_name, d in self.peddy_data.items():
            if 'PC1_het_check' in d and 'PC2_het_check' in d:
                data[s_name] = {
                    'x': d['PC1_het_check'],
                    'y': d['PC2_het_check']
                }
                try:
                    data[s_name]['color'] = ancestry_colors.get(d['ancestry-prediction'], default_color)
                except KeyError:
                    pass

        pconfig = {
            'id': 'peddy_pca_plot',
            'title': 'Peddy: PCA Plot',
            'xlab': 'PC1',
            'ylab': 'PC2',
            'marker_size': 5,
            'marker_line_width': 0
        }

        if len(data) > 0:
            self.add_section (
                name = 'PCA Plot',
                anchor = 'peddy-pca-plot',
                plot = scatter.plot(data, pconfig)
            )

    def peddy_relatedness_plot(self):
        data = dict()
        for s_name, d in self.peddy_data.items():
            if 'ibs0_ped_check' in d and 'ibs2_ped_check' in d:
                data[s_name] = {
                    'x': d['ibs0_ped_check'],
                    'y': d['ibs2_ped_check']
                }
            if 'rel_ped_check' in d:
                if d['rel_ped_check'] < 0.25:
                    data[s_name]['color'] = 'rgba(109, 164, 202, 0.9)'
                elif d['rel_ped_check'] < 0.5:
                    data[s_name]['color'] = 'rgba(250, 160, 81, 0.8)'
                else:
                    data[s_name]['color'] = 'rgba(43, 159, 43, 0.8)'

        pconfig = {
            'id': 'peddy_relatedness_plot',
            'title': 'Peddy: Relatedness Plot',
            'xlab': 'IBS0 (no alleles shared)',
            'ylab': 'IBS2 (both alleles shared)',
        }

        if len(data) > 0:
            self.add_section (
                name = 'Relatedness',
                anchor = 'peddy-relatedness-plot',
                description = """Shared allele rates between sample pairs. Points are coloured by degree of relatedness:
                <span style="color: #6DA4CA;">less than 0.25</span>,
                <span style="color: #FAA051;">0.25 - 0.5</span>,
                <span style="color: #2B9F2B;">greather than 0.5</span>.""",
                plot = scatter.plot(data, pconfig)
            )

    def peddy_het_check_plot(self):
        """plot the het_check scatter plot"""
        # empty dictionary to add sample names, and dictionary of values
        data = {}

        # for each sample, and list in self.peddy_data
        for s_name, d in self.peddy_data.items():
            # check the sample contains the required columns
            if 'median_depth_het_check' in d and 'het_ratio_het_check' in d:
                # add sample to dictionary with value as a dictionary of points to plot
                data[s_name] = {
                    'x': d['median_depth_het_check'],
                    'y': d['het_ratio_het_check']
                }

        pconfig = {
            'id': 'peddy_het_check_plot',
            'title': 'Peddy: Het Check',
            'xlab': 'median depth',
            'ylab': 'proportion het calls',
        }

        if len(data) > 0:
            self.add_section (
                name = 'Het Check',
                description = "Proportion of sites that were heterozygous against median depth.",
                helptext = """
                A high proportion of heterozygous sites suggests contamination, a low proportion suggests consanguinity.

                See [the main peddy documentation](https://peddy.readthedocs.io/en/latest/output.html#het-check) for more details about the `het_check` command.
                """,
                anchor = 'peddy-hetcheck-plot',
                plot = scatter.plot(data, pconfig)
            )

    def peddy_sex_check_plot(self):
        data = {}
        sex_index = {"female": 0, "male": 1, "unknown": 2}

        for s_name, d in self.peddy_data.items():
            if 'sex_het_ratio' in d and 'ped_sex_sex_check' in d:
                data[s_name] = {
                    'x': sex_index.get(d['ped_sex_sex_check'], 2),
                    'y': d["sex_het_ratio"]
                }

        pconfig = {
            'id': 'peddy_sex_check_plot',
            'title': 'Peddy: Sex Check',
            'xlab': 'Sex From Ped',
            'ylab': 'Sex Het Ratio',
            'categories': ["Female", "Male", "Unknown"]
        }

        if len(data) > 0:
            self.add_section(
                name = 'Sex Check',
                description = "Predicted sex against heterozygosity ratio",
                helptext = """
                Higher values of Sex Het Ratio suggests the sample is female, low values suggest male.

                See [the main peddy documentation](http://peddy.readthedocs.io/en/latest/#sex-check) for more details about the `het_check` command.
                """,
                anchor='peddy-sexcheck-plot',
                plot=scatter.plot(data, pconfig)
            )



""" MultiQC module to parse output from ngs-disambiguate. """

from collections import OrderedDict
import logging

from multiqc.modules.base_module import BaseMultiqcModule
from multiqc.plots import bargraph


log = logging.getLogger(__name__)


class MultiqcModule(BaseMultiqcModule):

    def __init__(self):
        # Initialise the parent object
        super(MultiqcModule, self).__init__(
            name='Disambiguate',
            anchor='disambiguate',
            href="https://github.com/AstraZeneca-NGS/disambiguate",
            info=("is an algorithm for disambiguating reads aligned "
                  "to two different species (e.g. human and mouse)."))

        self.data = {}

        for summary_file in self.find_log_files('disambiguate'):
            summ_data = self.parse_summary(summary_file['f'])

            for sample, counts in summ_data.items():
                # Clean sample name.
                sample = self.clean_s_name(sample, summary_file['root'])

                # Check for duplicates.
                if sample in self.data:
                    log.debug('Duplicate sample name found! Overwriting: %s',
                              sample)

                # Add to data and add data source.
                self.data[sample] = counts
                self.add_data_source(summary_file, s_name=sample)

        self.data = self.ignore_samples(self.data)

        if len(self.data) == 0:
            raise UserWarning

        log.info("Found %d reports", len(self.data))

        self.add_stats_table()
        self.add_stats_plot()
        self.write_stats_to_file()


    def parse_summary(self, contents):
        """Parses summary file into a dictionary of counts."""

        lines = contents.strip().split('\n')

        data = {}
        for row in lines[1:]:
            split = row.strip().split('\t')

            sample = split[0]

            data[sample] = {
                'species_a': int(split[1]),
                'species_b': int(split[2]),
                'ambiguous': int(split[3])
            }

        return data


    def add_stats_table(self):
        """Adds stats to general table."""

        totals = {sample: sum(counts.values())
                  for sample, counts in self.data.items()}

        percentages = {sample: {k: (v / totals[sample]) * 100
                                for k, v in counts.items()}
                       for sample, counts in self.data.items()}

        headers = {
            'species_a': {
                'title': '% Species a',
                'description': 'Percentage of reads mapping to species a',
                'max': 100,
                'min': 0,
                'suffix': '%',
                'scale': 'YlGn'
            }
        }

        self.general_stats_addcols(percentages, headers)


    def write_stats_to_file(self):
        """Writes stats to data file."""
        self.write_data_file(self.data, 'multiqc_disambiguate')


    def add_stats_plot(self):
        """Plots alignment stats as bargraph."""

        keys = OrderedDict()
        keys['species_a'] = {'color': '#437bb1', 'name': 'Species a'}
        keys['species_b'] = {'color': '#b1084c', 'name': 'Species b'}
        keys['ambiguous'] = {'color': '#333333', 'name': 'Ambiguous'}

        plot_config = {
            'id': "disambiguated_alignments",
            'title': "Disambiguate: Alignment Counts",
            'cpswitch_counts_label': "# Reads",
            'ylab': "# Reads"
        }

        self.add_section(
            plot=bargraph.plot(self.data, keys, plot_config)
        )



#!/usr/bin/env python

""" MultiQC module to parse output from sargasso """

from __future__ import print_function
from collections import OrderedDict
import logging

from multiqc import config
from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='sargasso',
        anchor='sargasso',
        href='http://statbio.github.io/Sargasso/',
        info="is a tool to separate mixed-species RNA-seq reads"\
             "according to their species of origin.")

         # Find and load any Sargasso reports
        self.sargasso_data = dict()
        self.sargasso_files = list()
        self.sargasso_keys = list() #header keys

        for f in self.find_log_files('sargasso'):
            self.parse_sargasso_logs(f)
            self.sargasso_files.append(f)

        # log.info('Removing ignored samples...')
        self.sargasso_data = self.ignore_samples(self.sargasso_data)

        if len(self.sargasso_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.sargasso_files)))

        # Write parsed report data to a file
        self.write_data_file(self.sargasso_data, 'multiqc_sargasso')

        # Basic Stats Table
        self.sargasso_stats_table()

        # Assignment bar plot
        self.add_section( plot = self.sargasso_chart() )

        # log.info('done')

    def parse_sargasso_logs(self, f):
        """ Parse the sargasso log file. """
        species_name = list()
        items = list()
        header = list()
        is_first_line = True
        for l in f['f'].splitlines():
            s = l.split(",")
            # Check that this actually is a Sargasso file
            if is_first_line and s[0]!='Sample':
                return None

            if len(s) < 7:
                continue
            if is_first_line:
                #prepare header
                is_first_line = False
                header = s
                for i in header[1:]:
                    #find out what species included
                    sname = i.split('-')[-1]
                    if sname not in species_name:
                        species_name.append(sname)
                    #find out what is being counted
                    kname = ("-".join(i.split('-')[-3:-1]))
                    if kname not in items:
                        items.append(kname)
            else:
                #start sample lines.
                sample_name = s.pop(0)

                chunk_by_species = [s[i:i + len(items)] for i in range(0, len(s), len(items))];
                for idx,v in enumerate(chunk_by_species):
                    #adding species name to the sample name for easy interpretation
                    new_sample_name = '_'.join([sample_name,species_name[idx]])

                    # Clean up sample name
                    new_sample_name = self.clean_s_name(new_sample_name, f['root'])

                    if new_sample_name in self.sargasso_data.keys():
                        log.debug("Duplicate sample name found! Overwriting: {}".format(new_sample_name))

                    try:
                        self.sargasso_data[new_sample_name] = dict(zip(items,map(int, v)))
                    except ValueError:
                        pass



        self.sargasso_keys = items

        for idx, f_name in enumerate(self.sargasso_data.keys()):



            # Reorganised parsed data for this sample
            # Collect total READ count number
            self.sargasso_data[f_name]['Total'] = 0;
            for key, value in list(self.sargasso_data[f_name].items()):   # iter on both keys and values
                if key.endswith("Reads"):
                    self.sargasso_data[f_name]['Total'] += value

            # Calculate the percent aligned if we can
            try:
                self.sargasso_data[f_name]['sargasso_percent_assigned'] = (float(self.sargasso_data[f_name]['Assigned-Reads'])/float(self.sargasso_data[f_name]['Total'])) * 100.0
            except (KeyError, ZeroDivisionError):
                pass


    def sargasso_stats_table(self):
        """ Take the parsed stats from the sargasso report and add them to the
        basic stats table at the top of the report """

        headers = OrderedDict()
        headers['sargasso_percent_assigned'] = {
            'title': '% Assigned',
            'description': 'Sargasso % Assigned reads',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'RdYlGn'
        }
        headers['Assigned-Reads'] = {
            'title': '{} Assigned'.format(config.read_count_prefix),
            'description': 'Sargasso Assigned reads ({})'.format(config.read_count_desc),
            'min': 0,
            'scale': 'PuBu',
            'modify': lambda x: float(x) * config.read_count_multiplier,
            'shared_key': 'read_count'
        }
        self.general_stats_addcols(self.sargasso_data, headers)


    def sargasso_chart (self):
        """ Make the sargasso plot """

        # Config for the plot
        config = {
            'id': 'sargasso_assignment_plot',
            'title': 'Sargasso: Assigned Reads',
            'ylab': '# Reads',
            'cpswitch_counts_label': 'Number of Reads'
        }

        #We only want to plot the READs at the moment
        return bargraph.plot(self.sargasso_data, [name for name in self.sargasso_keys if 'Reads' in name], config)



#!/usr/bin/env python

""" MultiQC submodule to parse output from fgbio ErrorRateByReadPosition """


from multiqc.plots import linegraph


def parse_reports(self):
    """Parser metric files for fgbio ErrorRateByReadPosition.

    Stores the per-read-per-position metrics into a data file and adds a section
    with a per-sample plot.
    """

    # slurp in all the data
    all_data = dict()
    error_rates = dict()
    y_max = 0.01  # default to 1%
    for f in self.find_log_files('fgbio/errorratebyreadposition', filehandles=True):
        fh = f['f']
        header = fh.readline().rstrip('\r\n').split('\t')
        if not header or header[0] != 'read_number':
            continue

        # slurp in the data for this sample
        s_name = f['s_name']
        s_data = dict()
        bases_total = 0
        errors = 0
        for line in fh:
            fields = line.rstrip('\r\n').split('\t')
            fields[1:4] = [int(field) for field in fields[1:4]]
            fields[4:] = [float(field) for field in fields[4:]]
            row_data = dict(zip(header, fields))
            read_number = row_data['read_number']
            position = row_data['position']
            if read_number not in s_data:
                s_data[read_number] = dict()
            s_data[read_number][position] = row_data
            if row_data['error_rate'] > y_max:
                y_max = row_data['error_rate']
            bases_total += row_data['bases_total']
            errors += row_data['errors']

        if s_data:
            all_data[s_name] = s_data
            error_rate = 0.0 if bases_total == 0 else errors / float(bases_total)
            error_rates[s_name] = {'error_rate': error_rate}


    # ignore samples
    all_data = self.ignore_samples(all_data)

    # if no data, then do nothing
    if not all_data:
        return 0

    # Write parsed data to a file
    self.write_data_file(all_data, 'multiqc_fgbio_ErrorRateByReadPosition_per_position')
    self.write_data_file(error_rates, 'multiqc_fgbio_ErrorRateByReadPosition_total')

    # Plot the data and add section
    pconfig = {
        'id': 'fgbio_ErrorRateByReadPosition',
        'title': 'Fgbio: Error Rate by Read Position',
        'ylab': 'Error rate',
        'xlab': 'Read Position',
        'xDecimals': False,
        'tt_label': '<b>read position {point.x}</b>: {point.y:.2f} %',
        'ymax': y_max,
        'ymin': 0,
        'data_labels': [
            {'name': 'Error Rate', 'ylab': 'Error Rate'},
            {'name': 'A > C', 'ylab': 'A to C error rate'},
            {'name': 'A > G', 'ylab': 'A to G error rate'},
            {'name': 'A > T', 'ylab': 'A to T error rate'},
            {'name': 'C > A', 'ylab': 'C to A error rate'},
            {'name': 'C > G', 'ylab': 'C to G error rate'},
            {'name': 'C > T', 'ylab': 'C to T error rate'},
        ]
    }

    # Build a list of linegraphs
    linegraph_data = [{}, {}, {}, {}, {}, {}, {}]
    linegraph_keys = ['error_rate', 'a_to_c_error_rate', 'a_to_g_error_rate', 'a_to_t_error_rate', 'c_to_a_error_rate', 'c_to_g_error_rate', 'c_to_t_error_rate']
    for s_name, s_data in all_data.items():
        for read_number, read_data in s_data.items():
            s_name = "%s_R%d" % (s_name, int(read_number) + 1)
            for lg, index in zip(linegraph_data, range(7)):
                lg[s_name] = dict((d['position'], d[linegraph_keys[index]]) for d in read_data.values())

    # add a section for the plot
    self.add_section (
        name = 'Error Rate by Read Position',
        anchor = 'fgbio-error-rate-by-read-position',
        description = 'Error rate by read position. Plot tabs show the error rates for specific substitution types.',
        helptext = '''
        The error rate by read position. Substitution types are collapsed based on the
        reference or expected base, with only six substitution types being reported:
        `A>C`, `A>G`, `A>T`, `C>A`, `C>G` and `C>T`.
        For example, `T>G` is grouped in with `A>C`.

        The following are reads / bases are excluded from the analysis:

        * Unmapped reads
        * Reads marked as failing vendor quality
        * Reads marked as duplicates (unless `--include-duplicates` was specified)
        * Secondary and supplemental records
        * Soft-clipped bases in records
        * Reads with MAPQ < `--min-mapping-quality` (default: `20`)
        * Bases with base quality < `--min-base-quality` (default: `0`)
        * Bases where either the read base or the reference base is non-ACGT
        ''',
        plot = linegraph.plot(linegraph_data, pconfig)
    )

    # Add to general stats table
    headers = {
        'error_rate': {
            'title': '% Error',
            'description': 'Percent error across all read positions',
            'min': 0,
            'max': 100.0,
            'scale': 'GnYlRd',
            'suffix': '%',
            'format': '{:,.2f}',
            'modify': lambda x: 100.0 * x
        }
    }
    self.general_stats_addcols(error_rates, headers)

    return len(all_data)



#!/usr/bin/env python

""" MultiQC module to parse output from HOMER tagdirectory """

import logging
import math
import os
import re
from multiqc.plots import bargraph, linegraph
from collections import OrderedDict

# Initialise the logger
log = logging.getLogger(__name__)


class TagDirReportMixin():


    def homer_tagdirectory(self):
        """ Find HOMER tagdirectory logs and parse their data """
        self.parse_gc_content()
        self.parse_re_dist()
        self.parse_tagLength_dist()
        self.parse_tagInfo_data()
        self.parse_FreqDistribution_data()
        self.homer_stats_table_tagInfo()

        return sum([len(v) for k, v in self.tagdir_data.items() if k != 'header'])


    def parse_gc_content(self):
        """parses and plots GC content and genome GC content files"""
        # Find and parse GC content:
        for f in self.find_log_files('homer/GCcontent', filehandles=True):
            # Get the s_name from the parent directory
            s_name = os.path.basename(f['root'])
            s_name = self.clean_s_name(s_name, f['root'])
            parsed_data = self.parse_twoCol_file(f)
            if parsed_data is not None:
                if s_name in self.tagdir_data['GCcontent']:
                    log.debug("Duplicate GCcontent sample log found! Overwriting: {}".format(s_name))

                self.add_data_source(f, s_name, section='GCcontent')
                self.tagdir_data['GCcontent'][s_name] = parsed_data

        ## get esimated genome content distribution:
        for f in self.find_log_files('homer/genomeGCcontent', filehandles=True):
            parsed_data = self.parse_twoCol_file(f)
            if parsed_data is not None:
                if s_name + "_genome" in self.tagdir_data['GCcontent']:
                    log.debug("Duplicate genome GCcontent sample log found! Overwriting: {}".format(s_name+ "_genome"))

                self.add_data_source(f, s_name + "_genome", section='GCcontent')
                self.tagdir_data['GCcontent'][s_name + "_genome"] = parsed_data

        self.tagdir_data['GCcontent'] = self.ignore_samples(self.tagdir_data['GCcontent'])

        if len(self.tagdir_data['GCcontent']) > 0:
            self.add_section (
                name = 'Per Sequence GC Content',
                anchor = 'homer_per_sequence_gc_content',
                description = 'This plot shows the distribution of GC content.',
                plot = self.GCcontent_plot()
            )


    def parse_re_dist(self):
        """parses and plots restriction distribution files"""
        # Find and parse homer restriction distribution reports
        for f in self.find_log_files('homer/RestrictionDistribution', filehandles=True):
            s_name = os.path.basename(f['root'])
            s_name = self.clean_s_name(s_name, f['root'])
            parsed_data = self.parse_restriction_dist(f)
            if parsed_data is not None:
                if s_name in self.tagdir_data['restriction']:
                    log.debug("Duplicate Restriction Distribution sample log found! Overwriting: {}".format(s_name))

                self.add_data_source(f, s_name, section='restriction')
                self.tagdir_data['restriction'][s_name] = parsed_data
                self.tagdir_data['restriction_norm'][s_name] = self.normalize(parsed_data)

        self.tagdir_data['restriction'] = self.ignore_samples(self.tagdir_data['restriction'])
        self.tagdir_data['restriction_norm'] = self.ignore_samples(self.tagdir_data['restriction_norm'])

        if len(self.tagdir_data['restriction']) > 0:
            self.add_section (
                name = 'Restriction Site Tag Dist',
                anchor = 'homer-restrictionDist',
                description = 'This plot shows the distribution of tags around restriction enzyme cut sites.',
                helptext = '''Hi-C data often involves the digestion of DNA using a restriction enzyme.
                    A good quality control for the experiment is the centering of reads around the
                    restriction enzyme cut site.''',
                plot = self.restriction_dist_chart()
            )


    def parse_tagLength_dist(self):
        """parses and plots tag length distribution files"""
        # Find and parse homer tag length distribution reports
        for f in self.find_log_files('homer/LengthDistribution', filehandles=True):
            s_name = os.path.basename(f['root'])
            s_name = self.clean_s_name(s_name, f['root'])
            parsed_data = self.parse_length_dist(f)
            if parsed_data is not None:
                if s_name in self.tagdir_data['length']:
                    log.debug("Duplicate Length Distribution sample log found! Overwriting: {}".format(s_name))
                self.add_data_source(f, s_name, section='length')
                self.tagdir_data['length'][s_name] = parsed_data

        self.tagdir_data['length'] = self.ignore_samples(self.tagdir_data['length'])
        if len(self.tagdir_data['length']) > 0:
            self.add_section (
                name = 'Tag Length Distribution',
                anchor = 'homer-tagLength',
                description = 'This plot shows the distribution of tag length.',
                helptext = 'This is a good quality control for tag length inputed into Homer.',
                plot = self.length_dist_chart()
            )


    def parse_tagInfo_data(self):
        """parses and plots taginfo files"""
        # Find and parse homer taginfo reports
        for f in self.find_log_files('homer/tagInfo', filehandles=True):
            s_name = os.path.basename(f['root'])
            s_name = self.clean_s_name(s_name, f['root'])
            parsed_data = self.parse_tag_info_chrs(f)
            if parsed_data is not None:
                if s_name in self.tagdir_data['taginfo_total']:
                    log.debug("Duplicate tag info sample log found! Overwriting: {}".format(s_name))
                self.add_data_source(f, s_name, section='taginfo')
                self.tagdir_data['taginfo_total'][s_name] = parsed_data[0]
                self.tagdir_data['taginfo_total_norm'][s_name] = self.normalize(parsed_data[0])
                self.tagdir_data['taginfo_uniq'][s_name] = parsed_data[1]
                self.tagdir_data['taginfo_uniq_norm'][s_name] = self.normalize(parsed_data[1])


        for f in self.find_log_files('homer/tagInfo', filehandles=True):
            s_name = os.path.basename(f['root'])
            s_name = self.clean_s_name(s_name, f['root'])
            ## collected tag_info data for general stats table and store under 'header'
            parsed_data = self.parse_tag_info(f)
            if parsed_data is not None:
                self.tagdir_data['header'][s_name] = parsed_data

        self.tagdir_data['taginfo_total'] = self.ignore_samples(self.tagdir_data['taginfo_total'])
        self.tagdir_data['taginfo_total_norm'] = self.ignore_samples(self.tagdir_data['taginfo_total_norm'])
        self.tagdir_data['taginfo_uniq'] = self.ignore_samples(self.tagdir_data['taginfo_uniq'])
        self.tagdir_data['taginfo_uniq_norm'] = self.ignore_samples(self.tagdir_data['taginfo_uniq_norm'])

        if len(self.tagdir_data['taginfo_total']) > 0:
            self.add_section (
                name = 'Chromosomal Coverage',
                anchor = 'homer-tagInfo',
                description = 'This plot shows the distribution of tags along chromosomes.',
                helptext = '''This is a good quality control for tag distribution and
                        could be a good indication of large duplications or deletions.''',
                plot = self.tag_info_chart()
            )


    def parse_FreqDistribution_data(self):
        """parses and plots taginfo files"""
        # Find and parse homer tag FreqDistribution_1000 reports
        for f in self.find_log_files('homer/FreqDistribution', filehandles=True):
            s_name = os.path.basename(f['root'])
            s_name = self.clean_s_name(s_name, f['root'])
            parsed_data = self.parse_FreqDist(f)
            if parsed_data is not None:
                if s_name in self.tagdir_data['FreqDistribution']:
                    log.debug("Duplicate Freq Distribution sample log found! Overwriting: {}".format(s_name))

                self.add_data_source(f, s_name, section='FreqDistribution')
                self.tagdir_data['FreqDistribution'][s_name] = parsed_data


        self.tagdir_data['FreqDistribution'] = self.ignore_samples(self.tagdir_data['FreqDistribution'])

        if len(self.tagdir_data['FreqDistribution']) > 0:
            self.add_section (
                name = 'Frequency Distribution',
                anchor = 'homer-FreqDistribution',
                description = 'This plot shows the distribution of distance between PE tags.',
                helptext = '''It is expected the the frequency of PE tags decays with
                    increasing distance between the PE tags. This plot gives an idea
                     of the proportion of short-range versus long-range interactions.''',
                plot = self.FreqDist_chart()
            )



    def homer_stats_table_tagInfo(self):
        """ Add core HOMER stats to the general stats table from tagInfo file"""

        if len(self.tagdir_data['header']) == 0:
            return None

        headers = OrderedDict()
        headers['UniqPositions'] = {
            'title': 'Uniq Pos',
            'description': 'Numer of Unique Di-Tags Passed Through HOMER',
            'format': '{:,.0f}',
            'modify': lambda x: x * 0.000001,
            'suffix': "M"

        }
        headers['TotalPositions'] = {
            'title': 'Total Pos',
            'description': 'Numer of Total Di-Tags Passed Through HOMER',
            'format': '{:,.0f}',
            'modify': lambda x: x * 0.000001,
            'suffix': "M"
        }
        headers['fragmentLengthEstimate'] = {
            'title': 'fragment Length',
            'description': 'Estimate of Fragnment Length',
            'format': '{:,.0f}'
        }
        headers['peakSizeEstimate'] = {
            'title': 'Peak Size',
            'description': 'Estimate of Peak Size',
            'format': '{:,.0f}'
        }
        headers['tagsPerBP'] = {
            'title': 'tagsPerBP',
            'description': 'average tags Per basepair',
            'format': '{:,.3f}',
        }
        headers['TagsPerPosition'] = {
            'title': 'averageTagsPerPosition',
            'description': 'Average Tags Per Position',
            'format': '{:,.2f}'
        }
        headers['averageTagLength'] = {
            'title': 'TagLength',
            'description': 'Average Tag Length',
            'format': '{:,.0f}'
        }
        headers['averageFragmentGCcontent'] = {
            'title': 'GCcontent',
            'description': 'Average Fragment GC content',
            'max': 1,
            'min': 0,
            'format': '{:,.2f}'
        }
        self.general_stats_addcols(self.tagdir_data['header'], headers, 'HOMER')


    def homer_stats_table_interChr(self):
        """ Add core HOMER stats to the general stats table from FrequencyDistribution file"""

        headers = OrderedDict()
        headers['InterChr'] = {
            'title': 'InterChr',
            'description': 'Fraction of Reads forming inter chromosomal interactions',
            'format': '{:,.4f}'
        }
        self.general_stats_addcols(self.tagdir_data['FreqDistribution'], headers, 'Homer-InterChr')


    def normalize(self, mydict, target=100):
       raw = sum(mydict.values())
       factor = target/raw
       return { key:value*factor for key,value in mydict.items() }


    def parse_twoCol_file(self, f):
        """ Parse HOMER tagdirectory GCcontent file. """
        parsed_data = dict()
        firstline = True
        for l in f['f']:
            if firstline:    #skip first line
                firstline = False
                continue
            s = l.split("\t")
            if len(s) > 1:
                k = float(s[0].strip())
                v = float(s[2].strip())
                parsed_data[k] = v

        return parsed_data



    def parse_restriction_dist(self, f):
        """ Parse HOMER tagdirectory petagRestrictionDistribution file. """
        parsed_data = dict()
        firstline = True
        for l in f['f']:
            if firstline:    #skip first line
                firstline = False
                continue
            s = l.split("\t")
            if len(s) > 1:
                nuc = float(s[0].strip())
                v1 = float(s[1].strip())
                v2 = float(s[2].strip())
                v = v1 + v2
                #parsed_data.update({nuc:v1})
                #parsed_data.update({nuc:v2})
                parsed_data.update({nuc:v})
        return parsed_data


    def parse_length_dist(self, f):
        """ Parse HOMER tagdirectory tagLengthDistribution file. """
        parsed_data = dict()
        firstline = True
        for l in f['f']:
            if firstline:    #skip first line
                firstline = False
                continue
            s = l.split("\t")
            if len(s) > 1:
                k = float(s[0].strip())
                v = float(s[1].strip())
                parsed_data[k] = v

        return parsed_data


    def parse_tag_info(self, f):
        """ Parse HOMER tagdirectory taginfo.txt file to extract statistics in the first 11 lines. """
        # General Stats Table
        tag_info = dict()
        for l in f['f']:
            s = l.split("=")
            if len(s) > 1:
                if s[0].strip() == 'genome':
                    ss = s[1].split("\t")
                    if len(ss) > 2:
                        tag_info['genome'] = ss[0].strip()
                        try:
                            tag_info['UniqPositions'] = float(ss[1].strip())
                            tag_info['TotalPositions'] = float(ss[2].strip())
                        except:
                            tag_info['UniqPositions'] = ss[1].strip()
                            tag_info['TotalPositions'] = ss[2].strip()
                try:
                    tag_info[s[0].strip()] = float(s[1].strip())
                except ValueError:
                    tag_info[s[0].strip()] = s[1].strip()
        return tag_info




    def parse_tag_info_chrs(self, f, convChr=True):
        """ Parse HOMER tagdirectory taginfo.txt file to extract chromosome coverage. """
        parsed_data_total = OrderedDict()
        parsed_data_uniq = OrderedDict()
        remove = ["hap", "random", "chrUn", "cmd", "EBV", "GL", "NT_"]
        for l in f['f']:
            s = l.split("\t")
            key = s[0].strip()
            # skip header
            if '=' in l or len(s) != 3:
                continue
            if convChr:
                if any(x in key for x in remove):
                    continue
            try:
                vT = float(s[1].strip())
                vU = float(s[2].strip())
            except ValueError:
                continue

            parsed_data_total[key] = vT
            parsed_data_uniq[key] = vU

        return [parsed_data_total, parsed_data_uniq]


    def parse_FreqDist(self, f):
        """ Parse HOMER tagdirectory petag.FreqDistribution_1000 file. """
        parsed_data = dict()
        firstline = True
        for l in f['f']:
            if firstline:
                firstline = False
                continue
            else:
                s = l.split("\t")
                if len(s) > 1:
                    k = s[0].strip()
                    if k.startswith("More than "):
                        k = re.sub("More than ", "", k)

                    k = float(k)
                    v = float(s[1].strip())
                    parsed_data[k] = v
        return parsed_data

    def parse_FreqDist_interChr(self, f):
        """ Parse HOMER tagdirectory petag.FreqDistribution_1000 file to get inter-chromosomal interactions. """
        parsed_data = dict()
        firstline = True
        for l in f['f']:
            if firstline:
                firstline = False
                interChr = float(re.sub("\)", "", l.split(":")[1]))
            else:
                break
        parsed_data['interChr'] = interChr
        return parsed_data


    def restriction_dist_chart (self):

        """ Make the petagRestrictionDistribution plot """

        pconfig = {
            'id': 'petagRestrictionDistribution',
            'title': 'HOMER: Restriction Distribution',
            'ylab': 'Reads',
            'xlab': 'Distance from cut site (bp)',
            'data_labels': [
                        {'name': 'Number of Tags'},
                        {'name': 'Percenatge'}
                            ]
       }
        datasets = [
            self.tagdir_data['restriction'],
            self.tagdir_data['restriction_norm']
        ]

        return linegraph.plot(datasets, pconfig)




    def length_dist_chart (self):

        """ Make the tagLengthDistribution plot """

        pconfig = {
            'id': 'tagLengthDistribution',
            'cpswitch': True,
            'title': 'HOMER: Tag Length Distribution',
            'ylab': 'Fraction of Tags',
            'xlab': 'Tag Length (bp)'
        }
        return linegraph.plot(self.tagdir_data['length'], pconfig)



    def GCcontent_plot (self):
        """ Create the HTML for the Homer GC content plot """

        pconfig = {
            'id': 'homer-tag-directory-gc-content',
            'title': 'HOMER: Tag Directory Per Sequence GC Content',
            'smooth_points': 200,
            'smooth_points_sumcounts': False,
            'ylab': 'Normalized Count',
            'xlab': '% GC',
            'ymin': 0,
            'xmax': 1,
            'xmin': 0,
            'yDecimals': True,
            'tt_label': '<b>{point.x}% GC</b>: {point.y}'
        }
        return linegraph.plot(self.tagdir_data['GCcontent'], pconfig)


    def tag_info_chart (self):

        """ Make the taginfo.txt plot """

        ## TODO: human chrs on hg19. How will this work with GRCh genome or other, non human, genomes?
        # nice if they are ordered by size
        ucsc = ["chr" + str(i) for i in range(1,23)].append([ "chrX", "chrY", "chrM"])
        ensembl = list(range(1,23)).append([ "X", "Y", "MT"])
        pconfig = {
            'id': 'tagInfo',
            'title': 'HOMER: Tag Info Distribution',
            'ylab': 'Tags',
            'cpswitch_counts_label': 'Number of Tags'
        }

        ## check if chromosomes starts with "chr" (UCSC) or "#" (ensembl)
        sample1 = next(iter(self.tagdir_data['taginfo_total']))
        chrFormat = next(iter(self.tagdir_data['taginfo_total'][sample1]))

        if ("chr" in chrFormat):
            chrs = ucsc
        else:
            chrs = ensembl

        return bargraph.plot(self.tagdir_data['taginfo_total'], chrs, pconfig)


    def FreqDist_chart (self):
        """ Make the petag.FreqDistribution_1000 plot """
        # Take a log of the data before plotting so that we can
        # reduce the number of points to plot evenly
        pdata = {}
        for idx, s_name in enumerate(self.tagdir_data['FreqDistribution']):
            pdata[s_name] = {}
            for x, y in self.tagdir_data['FreqDistribution'][s_name].items():
                try:
                    pdata[s_name][math.log(float(x))] = y
                except ValueError:
                    pass
        pconfig = {
            'id': 'FreqDistribution',
            'title': 'HOMER: Frequency Distribution',
            'ylab': 'Fraction of Reads',
            'xlab': 'Log10(Distance between regions)',
            'data_labels': ['Reads', 'Percent'],
            'smooth_points': 500,
            'smooth_points_sumcounts': False,
            'yLog' : True
        }
        return linegraph.plot(pdata, pconfig)



#!/usr/bin/env python

""" MultiQC functions to plot a beeswarm group """

import logging
import random

from multiqc.utils import config, report
from multiqc.plots import table_object

logger = logging.getLogger(__name__)

letters = 'abcdefghijklmnopqrstuvwxyz'

def plot (data, headers=None, pconfig=None):
    """ Helper HTML for a beeswarm plot.
    :param data: A list of data dicts
    :param headers: A list of Dicts / OrderedDicts with information
                    for the series, such as colour scales, min and
                    max values etc.
    :return: HTML string
    """
    if headers is None:
        headers = []
    if pconfig is None:
        pconfig = {}

    # Allow user to overwrite any given config for this plot
    if 'id' in pconfig and pconfig['id'] and pconfig['id'] in config.custom_plot_config:
        for k, v in config.custom_plot_config[pconfig['id']].items():
            pconfig[k] = v

    # Make a datatable object
    dt = table_object.datatable(data, headers, pconfig)

    return make_plot( dt )


def make_plot(dt):

    bs_id = dt.pconfig.get('id', 'table_{}'.format(''.join(random.sample(letters, 4))) )

    # Sanitise plot ID and check for duplicates
    bs_id = report.save_htmlid(bs_id)

    categories = []
    s_names = []
    data = []
    for idx, hs in enumerate(dt.headers):
        for k, header in hs.items():

            bcol = 'rgb({})'.format(header.get('colour', '204,204,204'))

            categories.append({
                'namespace': header['namespace'],
                'title': header['title'],
                'description': header['description'],
                'max': header['dmax'],
                'min': header['dmin'],
                'suffix': header.get('suffix', ''),
                'decimalPlaces': header.get('decimalPlaces', '2'),
                'bordercol': bcol
            });

            # Add the data
            thisdata = []
            these_snames = []
            for (s_name, samp) in dt.data[idx].items():
                if k in samp:

                    val = samp[k]

                    if 'modify' in header and callable(header['modify']):
                        val = header['modify'](val)

                    thisdata.append(val)
                    these_snames.append(s_name)

            data.append(thisdata)
            s_names.append(these_snames)

    if len(s_names) == 0:
        logger.warning('Tried to make beeswarm plot, but had no data')
        return '<p class="text-danger">Error - was not able to plot data.</p>'

    # Plot HTML
    html = """<div class="hc-plot-wrapper">
        <div id="{bid}" class="hc-plot not_rendered hc-beeswarm-plot"><small>loading..</small></div>
    </div>""".format(bid=bs_id)

    report.num_hc_plots += 1

    report.plot_data[bs_id] = {
        'plot_type': 'beeswarm',
        'samples': s_names,
        'datasets': data,
        'categories': categories
    }

    return html



#!/usr/bin/env python

""" MultiQC module to parse output from KAT """
import logging
from collections import OrderedDict
import json

from multiqc.modules.base_module import BaseMultiqcModule
from multiqc.plots import table

# Initialise the logger
log = logging.getLogger(__name__)


class MultiqcModule(BaseMultiqcModule):
    def __init__(self):
        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='K-mer Analysis Toolkit', anchor='kat',
                                            href="https://github.com/TGAC/KAT",
                                            info="is an toolkit for analysing sequencing data via its k-mer spectra.")

        # Find and load any KAT dist analysis reports
        self.kat_data = dict()
        for c_file in self.find_log_files('kat'):
            s_name = self.clean_s_name(c_file['s_name'].replace(".dist_analysis", ""), c_file['root'])
            content = json.loads(c_file['f'])
            self.kat_data[s_name] = self.parse_kat_report(content)

        # Filter to strip out ignored sample names
        self.kat_data = self.ignore_samples(self.kat_data)

        if len(self.kat_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.kat_data)))

        # Write parsed report data to a file
        self.write_data_file(self.kat_data, 'multiqc_kat')

        headers = OrderedDict()
        headers["kmer_peaks"] = {
            'title': '# of Kmer Peaks',
            'description': "Number of peaks identified in the K-mer spectra",
            'scale': False,
            'format': '{:,.0f}'
        }
        headers["gc_peaks"] = {
            'title': '# of GC Peaks',
            'description': "Number of peaks identified in the GC distribution",
            'scale': False,
            'format': '{:,.0f}'
        }
        headers["est_genome_size"] = {
            'title': 'Est. genome Size',
            'description': "Estimated Genome Size based on K-mer spectra",
            'scale': "BuPu",
            'format': '{:,.0f}'
        }
        headers["mean_kmer_freq"] = {
            'title': 'Mean K-mer Freq.',
            'description': "Mean K-mer Frequency, provides an estimate of sequencing coverage",
            'scale': "Greens",
            'format': '{:,.0f}',
            'suffix': 'x'
        }

        kat_config = {
            'namespace': 'KAT',
        }

        # Basic Stats Table
        self.add_section(
            name='KAT Distribution Analysis',
            anchor='kat-first',
            description='Table showing k-mer coverage distributions and if available GC distributions',
            helptext="This table can give a quick idea of potential contaminants that can be identified via unexpected numbers of k-mer or gc peaks in the data",
            plot=table.plot(self.kat_data, headers, kat_config)
        )

    def parse_kat_report(self, content):
        table_data = {}
        if "gc" in content and "coverage" in content:
            # GCP
            table_data['kmer_peaks'] = content['coverage']['nb_peaks']
            table_data['mean_kmer_freq'] = content['coverage']['mean_freq']
            table_data['est_genome_size'] = content['coverage']['est_genome_size']
            table_data['gc_peaks'] = content['gc']['nb_peaks']
        elif "main_dist" in content:
            # Spectra CN
            table_data['kmer_peaks'] = content['main_dist']['nb_peaks']
            table_data['mean_kmer_freq'] = content['main_dist']['mean_freq']
            table_data['est_genome_size'] = content['main_dist']['est_genome_size']
            table_data['gc_peaks'] = 0
        elif "k" in content:
            # Hist
            table_data['kmer_peaks'] = content['nb_peaks']
            table_data['mean_kmer_freq'] = content['mean_freq']
            table_data['est_genome_size'] = content['est_genome_size']
            table_data['gc_peaks'] = 0
        else:
            log.error("Unexpected JSON configuration")

        return table_data



#!/usr/bin/env python
from __future__ import print_function

import re
from collections import OrderedDict, defaultdict
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
import logging
log = logging.getLogger(__name__)


NAMESPACE = 'Ploidy estimation'


class DragenPloidyEstimationMetrics(BaseMultiqcModule):
    def add_ploidy_estimation_metrics(self):
        data_by_sample = dict()

        for f in self.find_log_files('dragen/ploidy_estimation_metrics'):
            data = parse_ploidy_estimation_metrics_file(f)
            if f['s_name'] in data_by_sample:
                log.debug('Duplicate sample name found! Overwriting: {}'.format(f['s_name']))
            self.add_data_source(f, section='stats')
            data_by_sample[f['s_name']] = data

        # Filter to strip out ignored sample names:
        data_by_sample = self.ignore_samples(data_by_sample)
        if not data_by_sample:
            return set()

        headers = OrderedDict()
        headers['Ploidy estimation'] = {
            'title': 'Sex',
            'description': 'Sex chromosome ploidy estimation (XX, XY, X0, 00, etc.)',
            'scale': 'Set3',
        }
        self.general_stats_addcols(data_by_sample, headers, namespace=NAMESPACE)
        return data_by_sample.keys()


def parse_ploidy_estimation_metrics_file(f):
    """
    T_SRR7890936_50pc.ploidy_estimation_metrics.csv

    PLOIDY ESTIMATION,,Autosomal median coverage,55.63
    PLOIDY ESTIMATION,,X median coverage,27.44
    PLOIDY ESTIMATION,,Y median coverage,0.00
    PLOIDY ESTIMATION,,X median / Autosomal median,0.49
    PLOIDY ESTIMATION,,Y median / Autosomal median,0.00
    PLOIDY ESTIMATION,,Ploidy estimation,X0
    """

    f['s_name'] = re.search(r'(.*).ploidy_estimation_metrics.csv', f['fn']).group(1)

    data = defaultdict(dict)

    for line in f['f'].splitlines():
        _, _, metric, stat = line.split(',')
        try:
            stat = float(stat)
        except ValueError:
            pass
        data[metric] = stat

    return data



#!/usr/bin/env python

""" MultiQC submodule to parse output from RSeQC read_distribution.py
http://rseqc.sourceforge.net/#read-distribution-py """

from collections import OrderedDict
import logging
import re

from multiqc.plots import bargraph

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """ Find RSeQC read_distribution reports and parse their data """

    # Set up vars
    self.read_dist = dict()
    first_regexes = {
        'total_reads': r"Total Reads\s+(\d+)\s*",
        'total_tags': r"Total Tags\s+(\d+)\s*",
        'total_assigned_tags': r"Total Assigned Tags\s+(\d+)\s*",
    }
    second_regexes = {
        'cds_exons': r"CDS_Exons\s+(\d+)\s+(\d+)\s+([\d\.]+)\s*",
        '5_utr_exons': r"5'UTR_Exons\s+(\d+)\s+(\d+)\s+([\d\.]+)\s*",
        '3_utr_exons': r"3'UTR_Exons\s+(\d+)\s+(\d+)\s+([\d\.]+)\s*",
        'introns': r"Introns\s+(\d+)\s+(\d+)\s+([\d\.]+)\s*",
        'tss_up_1kb': r"TSS_up_1kb\s+(\d+)\s+(\d+)\s+([\d\.]+)\s*",
        'tss_up_5kb': r"TSS_up_5kb\s+(\d+)\s+(\d+)\s+([\d\.]+)\s*",
        'tss_up_10kb': r"TSS_up_10kb\s+(\d+)\s+(\d+)\s+([\d\.]+)\s*",
        'tes_down_1kb': r"TES_down_1kb\s+(\d+)\s+(\d+)\s+([\d\.]+)\s*",
        'tes_down_5kb': r"TES_down_5kb\s+(\d+)\s+(\d+)\s+([\d\.]+)\s*",
        'tes_down_10kb': r"TES_down_10kb\s+(\d+)\s+(\d+)\s+([\d\.]+)\s*",
    }

    # Go through files and parse data using regexes
    for f in self.find_log_files('rseqc/read_distribution'):
        d = dict()
        for k, r in first_regexes.items():
            r_search = re.search(r, f['f'], re.MULTILINE)
            if r_search:
                d[k] = int(r_search.group(1))
        for k, r in second_regexes.items():
            r_search = re.search(r, f['f'], re.MULTILINE)
            if r_search:
                d['{}_total_bases'.format(k)] = int(r_search.group(1))
                d['{}_tag_count'.format(k)] = int(r_search.group(2))
                d['{}_tags_kb'.format(k)] = float(r_search.group(3))

        d['other_intergenic_tag_count'] = d['total_tags']-d['total_assigned_tags']

        # Calculate some percentages for parsed file
        if 'total_tags' in d:
            t = float(d['total_tags'])
            pcts = dict()
            for k in d:
                if k.endswith('_tag_count'):
                    pk = '{}_tag_pct'.format(k[:-10])
                    pcts[pk] = (float(d[k]) / t)*100.0
            d.update(pcts)

        if len(d) > 0:
            if f['s_name'] in self.read_dist:
                log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
            self.add_data_source(f, section='read_distribution')
            self.read_dist[f['s_name']] = d

    # Filter to strip out ignored sample names
    self.read_dist = self.ignore_samples(self.read_dist)

    if len(self.read_dist) > 0:

        # Write to file
        self.write_data_file(self.read_dist, 'multiqc_rseqc_read_distribution')

        # Plot bar graph of groups
        keys = OrderedDict()
        keys['cds_exons_tag_count'] = {'name': "CDS_Exons"}
        keys['5_utr_exons_tag_count'] = {'name': "5'UTR_Exons"}
        keys['3_utr_exons_tag_count'] = {'name': "3'UTR_Exons"}
        keys['introns_tag_count'] = {'name': "Introns"}
        keys['tss_up_1kb_tag_count'] = {'name': "TSS_up_1kb"}
        keys['tss_up_5kb_tag_count'] = {'name': "TSS_up_5kb"}
        keys['tss_up_10kb_tag_count'] = {'name': "TSS_up_10kb"}
        keys['tes_down_1kb_tag_count'] = {'name': "TES_down_1kb"}
        keys['tes_down_5kb_tag_count'] = {'name': "TES_down_5kb"}
        keys['tes_down_10kb_tag_count'] = {'name': "TES_down_10kb"}
        keys['other_intergenic_tag_count'] = {'name': "Other_intergenic"}

        # Config for the plot
        pconfig = {
            'id': 'rseqc_read_distribution_plot',
            'title': 'RSeQC: Read Distribution',
            'ylab': '# Tags',
            'cpswitch_counts_label': 'Number of Tags',
            'cpswitch_c_active': False
        }

        self.add_section (
            name = 'Read Distribution',
            anchor = 'rseqc-read_distribution',
            description = '<a href="http://rseqc.sourceforge.net/#read-distribution-py" target="_blank">Read Distribution</a>' \
                " calculates how mapped reads are distributed over genome features.",
            plot = bargraph.plot(self.read_dist, keys, pconfig)
        )

    # Return number of samples found
    return len(self.read_dist)



#!/usr/bin/env python

""" MultiQC module to parse output from Conpair """

from __future__ import print_function
import logging
import re

from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """
    Conpair module class.
    """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='Conpair', anchor='conpair',
        href='https://github.com/nygenome/Conpair',
        info="is a fast and robust method dedicated for human tumor-normal "\
             "studies to perform concordance verification, as well as "\
             "cross-individual contamination level estimation in "\
             "whole-genome and whole-exome sequencing experiments.")

        self.conpair_data = dict()

        for f in self.find_log_files('conpair/concordance'):
            self.parse_conpair_logs(f)

        for f in self.find_log_files('conpair/contamination'):
            self.parse_conpair_logs(f)

        # Filter to strip out ignored sample names
        self.conpair_data = self.ignore_samples(self.conpair_data)

        if len(self.conpair_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.conpair_data)))

        # Write parsed report data to a file
        self.write_data_file(self.conpair_data, 'multiqc_conpair')

        # Basic Stats Table
        self.conpair_general_stats_table()


    def parse_conpair_logs(self, f):
        """ Go through log file looking for conpair concordance or contamination output
        One parser to rule them all. """

        conpair_regexes = {
            'concordance_concordance': r"Concordance: ([\d\.]+)%",
            'concordance_used_markers': r"Based on (\d+)/\d+ markers",
            'concordance_total_markers': r"Based on \d+/(\d+) markers",
            'concordance_marker_threshold': r"\(coverage per marker threshold : (\d+) reads\)",
            'concordance_min_mapping_quality': r"Minimum mappinq quality: (\d+)",
            'concordance_min_base_quality': r"Minimum base quality: (\d+)",
            'contamination_normal': r"Normal sample contamination level: ([\d\.]+)%",
            'contamination_tumor': r"Tumor sample contamination level: ([\d\.]+)%"
        }

        parsed_data = {}
        for k, r in conpair_regexes.items():
            match = re.search(r, f['f'])
            if match:
                parsed_data[k] = float(match.group(1))

        def _cp_type(data):
            if 'concordance_concordance' in parsed_data:
                return 'concordance'
            elif 'contamination_normal' in parsed_data:
                return 'contamination'

        if len(parsed_data) > 0:
            if f['s_name'] in self.conpair_data:
                if(_cp_type(self.conpair_data[f['s_name']]) == _cp_type(parsed_data)):
                    log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
            else:
                self.conpair_data[f['s_name']] = dict()
            self.add_data_source(f, section=_cp_type(parsed_data))
            self.conpair_data[f['s_name']].update(parsed_data)


    def conpair_general_stats_table(self):
        """ Take the parsed stats from the Conpair report and add it to the
        basic stats table at the top of the report """

        headers = {}
        headers['concordance_concordance'] = {
            'title': 'Concordance',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'format': '{:,.2f}',
            'scale': 'RdYlGn'
        }
        headers['contamination_normal'] = {
            'title': 'N Contamination',
            'description': 'Normal sample contamination level',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'format': '{:,.3f}',
            'scale': 'RdYlBu-rev'
        }
        headers['contamination_tumor'] = {
            'title': 'T Contamination',
            'description': 'Tumor sample contamination level',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'format': '{:,.3f}',
            'scale': 'RdYlBu-rev'
        }
        self.general_stats_addcols(self.conpair_data, headers)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" MultiQC submodule to parse output from GATK BaseRecalibrator """

import logging
from collections import namedtuple
from multiqc.plots import linegraph

# Initialise the logger
log = logging.getLogger(__name__)

RecalTableType = namedtuple("RecalTableType", "pre_recalibration post_recalibration".split())
recal_table_type = RecalTableType(0, 1)

class BaseRecalibratorMixin():
    def parse_gatk_base_recalibrator(self):
        """ Find GATK BaseRecalibrator logs and parse their data """

        report_table_headers = {
            '#:GATKTable:Arguments:Recalibration argument collection values used in this run': 'arguments',
            '#:GATKTable:Quantized:Quality quantization map': 'quality_quantization_map',
            '#:GATKTable:RecalTable0:': 'recal_table_0',
            '#:GATKTable:RecalTable1:': 'recal_table_1',
            '#:GATKTable:RecalTable2:': 'recal_table_2',
        }
        samples_kept = {rt_type: set() for rt_type in recal_table_type}
        self.gatk_base_recalibrator = {
            recal_type: {
                table_name: {} for table_name in report_table_headers.values()
            } for recal_type in recal_table_type
        }

        for f in self.find_log_files('gatk/base_recalibrator', filehandles=True):

            # Check that we're not ignoring this sample name
            if self.is_ignore_sample(f['s_name']):
                continue

            parsed_data = self.parse_report(f['f'].readlines(), report_table_headers)
            rt_type = determine_recal_table_type(parsed_data)
            if len(parsed_data) > 0:
                if f['s_name'] in samples_kept[rt_type]:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
                samples_kept[rt_type].add(f['s_name'])

                self.add_data_source(f, section='base_recalibrator')
                for table_name, sample_tables in parsed_data.items():
                    self.gatk_base_recalibrator[rt_type][table_name][f['s_name']] = sample_tables

        # Filter to strip out ignored sample names. Again.
        for rt_type in recal_table_type:
            for table_name, sample_tables in self.gatk_base_recalibrator[rt_type].items():
                self.gatk_base_recalibrator[rt_type][table_name] = self.ignore_samples(
                    sample_tables)

        n_reports_found = sum([len(samples_kept[rt_type]) for rt_type in recal_table_type])

        if n_reports_found > 0:
            log.info("Found {} BaseRecalibrator reports".format(n_reports_found))

            self.add_quality_score_vs_no_of_observations_section()

        return n_reports_found

    def add_quality_score_vs_no_of_observations_section(self):
        """ Add a section for the quality score vs number of observations line plot """

        sample_data = []
        data_labels = []

        # Loop through the different data types
        for rt_type_name, rt_type in recal_table_type._asdict().items():
            sample_tables = self.gatk_base_recalibrator[rt_type]['quality_quantization_map']
            if len(sample_tables) == 0:
                continue

            count_data = {}
            pct_data = {}
            for sample, table in sample_tables.items():
                count_data[sample] = {}
                pct_data[sample] = {}

                # Get the total count for this sample
                sample_y_sum = sum( int(y) for y in table['Count'] )

                # Collect the data for the plots
                for x, y in zip(table['QualityScore'], table['Count']):
                    # Quality score counts
                    count_data[sample][int(x)] = int(y)
                    # Quality score percentages
                    try:
                        pct_data[sample][int(x)] = float(y) / sample_y_sum
                    except ZeroDivisionError:
                        pct_data[sample][int(x)] = 0

            # Append the datasets for this data type
            sample_data.append(count_data)
            sample_data.append(pct_data)

            # Build data label configs for this data type
            data_labels.append({
                'name': "{} Count".format(rt_type_name.capitalize().replace('_', '-')),
                'ylab': 'Count'
            })
            data_labels.append({
                'name': "{} Percent".format(rt_type_name.capitalize().replace('_', '-')),
                'ylab': 'Percent'
            })

        plot = linegraph.plot(
            sample_data,
            pconfig = {
                'title': "GATK: Observed Quality Score Counts",
                'id': 'gatk-base-recalibrator-quality-scores-plot',
                'xlab': 'Observed Quality Score',
                'ylab': 'Count',
                'xDecimals': False,
                'data_labels': data_labels
            })

        # Reported vs empirical quality scores
        self.add_section(
            name = 'Observed Quality Scores',
            anchor = 'gatk-base-recalibrator-quality-scores',
            description = (
                'This plot shows the distribution of base quality scores in each sample before and '
                'after base quality score recalibration (BQSR). Applying BQSR should broaden the '
                'distribution of base quality scores.'
            ),
            helptext = (
                'For more information see '
                '[the Broad\'s description of BQSR]'
                '(https://gatkforums.broadinstitute.org/gatk/discussion/44/base-quality-score-recalibration-bqsr)'
                '.'
            ),
            plot = plot,
        )


def determine_recal_table_type(parsed_data):
    arguments = dict(zip(parsed_data['arguments']['Argument'], parsed_data['arguments']['Value']))
    if arguments['recalibration_report'] == 'null':
        return recal_table_type.pre_recalibration
    else:
        return recal_table_type.post_recalibration



#!/usr/bin/env python

""" MultiQC datatable class, used by tables and beeswarm plots """

from collections import defaultdict, OrderedDict
import logging
import re

from multiqc.utils import config, report

logger = logging.getLogger(__name__)

class datatable (object):
    """ Data table class. Prepares and holds data and configuration
    for either a table or a beeswarm plot. """

    def __init__ (self, data, headers=None, pconfig=None):
        """ Prepare data for use in a table or plot """
        if headers is None:
            headers = []
        if pconfig is None:
            pconfig = {}

        # Given one dataset - turn it into a list
        if type(data) is not list:
            data = [data]
        if type(headers) is not list:
            headers = [headers]

        sectcols = ['55,126,184', '77,175,74', '152,78,163', '255,127,0', '228,26,28', '255,255,51', '166,86,40', '247,129,191', '153,153,153']
        shared_keys = defaultdict(lambda: dict())

        # Go through each table section
        for idx, d in enumerate(data):

            # Get the header keys
            try:
                keys = headers[idx].keys()
                assert len(keys) > 0
            except (IndexError, AttributeError, AssertionError):
                pconfig['only_defined_headers'] = False

            # Add header keys from the data
            if pconfig.get('only_defined_headers', True) is False:

                # Get the keys from the data
                keys = list()
                for samp in d.values():
                    for k in samp.keys():
                        if k not in keys:
                            keys.append(k)

                # If we don't have a headers dict for this data set yet, create one
                try:
                    headers[idx]
                except IndexError:
                    headers.append(list)
                    headers[idx] = OrderedDict()
                else:
                    # Convert the existing headers into an OrderedDict (eg. if parsed from a config)
                    od_tuples = [(key, headers[idx][key]) for key in headers[idx].keys()]
                    headers[idx] = OrderedDict(od_tuples)

                # Create empty header configs for each new data key
                for k in keys:
                    if k not in headers[idx]:
                        headers[idx][k] = {}

            # Ensure that keys are strings, not numeric
            keys = [str(k) for k in keys]
            for k in list(headers[idx].keys()):
                headers[idx][str(k)] = headers[idx].pop(k)
            # Ensure that all sample names are strings as well
            cdata = OrderedDict()
            for k,v in data[idx].items():
                cdata[str(k)] = v
            data[idx] = cdata
            for s_name in data[idx].keys():
                for k in list(data[idx][s_name].keys()):
                    data[idx][s_name][str(k)] = data[idx][s_name].pop(k)

            # Check that we have some data in each column
            empties = list()
            for k in keys:
                n = 0
                for samp in d.values():
                    if k in samp:
                        n += 1
                if n == 0:
                    empties.append(k)
            for k in empties:
                keys = [j for j in keys if j != k]
                del headers[idx][k]

            for k in keys:
                # Unique id to avoid overwriting by other datasets
                if 'rid' not in headers[idx][k]:
                    headers[idx][k]['rid'] = report.save_htmlid(re.sub(r'\W+', '_', k).strip().strip('_'))

                # Applying defaults presets for data keys if shared_key is set to base_count or read_count
                shared_key = headers[idx][k].get('shared_key', None)
                if shared_key in ['read_count', 'base_count']:
                    if shared_key == 'read_count':
                        multiplier = config.read_count_multiplier
                    else:
                        multiplier = config.base_count_multiplier
                    if headers[idx][k].get('modify') is None:
                        headers[idx][k]['modify'] = lambda x: x * multiplier
                    if headers[idx][k].get('min') is None:
                        headers[idx][k]['min'] = 0
                    if headers[idx][k].get('format') is None:
                        if multiplier == 1:
                            headers[idx][k]['format'] = '{:,.0f}'

                # Use defaults / data keys if headers not given
                headers[idx][k]['namespace']   = headers[idx][k].get('namespace', pconfig.get('namespace', ''))
                headers[idx][k]['title']       = headers[idx][k].get('title', k)
                headers[idx][k]['description'] = headers[idx][k].get('description', headers[idx][k]['title'])
                headers[idx][k]['scale']       = headers[idx][k].get('scale', pconfig.get('scale', 'GnBu'))
                headers[idx][k]['format']      = headers[idx][k].get('format', pconfig.get('format', '{:,.1f}'))
                headers[idx][k]['colour']      = headers[idx][k].get('colour', pconfig.get('colour', None))
                headers[idx][k]['hidden']      = headers[idx][k].get('hidden', pconfig.get('hidden', None))
                headers[idx][k]['max']         = headers[idx][k].get('max', pconfig.get('max', None))
                headers[idx][k]['min']         = headers[idx][k].get('min', pconfig.get('min', None))
                headers[idx][k]['ceiling']     = headers[idx][k].get('ceiling', pconfig.get('ceiling', None))
                headers[idx][k]['floor']       = headers[idx][k].get('floor', pconfig.get('floor', None))
                headers[idx][k]['minRange']    = headers[idx][k].get('minRange', pconfig.get('minRange', None))
                headers[idx][k]['shared_key']  = headers[idx][k].get('shared_key', pconfig.get('shared_key', None))
                headers[idx][k]['modify']      = headers[idx][k].get('modify', pconfig.get('modify', None))
                headers[idx][k]['placement']   = float( headers[idx][k].get('placement', 1000) )

                if headers[idx][k]['colour'] is None:
                    cidx = idx
                    while cidx >= len(sectcols):
                        cidx -= len(sectcols)
                    headers[idx][k]['colour'] = sectcols[cidx]

                # Overwrite hidden if set in user config
                for ns in config.table_columns_visible.keys():
                    # Make namespace key case insensitive
                    if ns.lower() == headers[idx][k]['namespace'].lower():

                        # First - if config value is a bool, set all module columns to that value
                        if isinstance(config.table_columns_visible[ns], bool):
                            headers[idx][k]['hidden'] = not config.table_columns_visible[ns]

                        # Not a bool, assume a dict of the specific column IDs
                        else:
                            try:
                                # Config has True = visibile, False = Hidden. Here we're setting "hidden" which is inverse
                                headers[idx][k]['hidden'] = not config.table_columns_visible[ns][k]
                            except KeyError:
                                pass

                # Also overwite placement if set in config
                try:
                    headers[idx][k]['placement'] = float(config.table_columns_placement[ headers[idx][k]['namespace'] ][k])
                except (KeyError, ValueError):
                    try:
                        headers[idx][k]['placement'] = float(config.table_columns_placement[ pconfig['id'] ][k])
                    except (KeyError, ValueError):
                        pass

                # Work out max and min value if not given
                setdmax = False
                setdmin = False
                try:
                    headers[idx][k]['dmax'] = float(headers[idx][k]['max'])
                except TypeError:
                    headers[idx][k]['dmax'] = 0
                    setdmax = True

                try:
                    headers[idx][k]['dmin'] = float(headers[idx][k]['min'])
                except TypeError:
                    headers[idx][k]['dmin'] = 0
                    setdmin = True

                # Figure out the min / max if not supplied
                if setdmax or setdmin:
                    for s_name, samp in data[idx].items():
                        try:
                            val = float(samp[k])
                            if callable(headers[idx][k]['modify']):
                                val = float(headers[idx][k]['modify'](val))
                            if setdmax:
                                headers[idx][k]['dmax'] = max(headers[idx][k]['dmax'], val)
                            if setdmin:
                                headers[idx][k]['dmin'] = min(headers[idx][k]['dmin'], val)
                        except ValueError:
                            val = samp[k] # couldn't convert to float - keep as a string
                        except KeyError:
                            pass # missing data - skip
                    # Limit auto-generated scales with floor, ceiling and minRange.
                    if headers[idx][k]['ceiling'] is not None and headers[idx][k]['max'] is None:
                        headers[idx][k]['dmax'] = min(headers[idx][k]['dmax'], float(headers[idx][k]['ceiling']))
                    if headers[idx][k]['floor'] is not None and headers[idx][k]['min'] is None:
                        headers[idx][k]['dmin'] = max(headers[idx][k]['dmin'], float(headers[idx][k]['floor']))
                    if headers[idx][k]['minRange'] is not None:
                        drange = headers[idx][k]['dmax'] - headers[idx][k]['dmin']
                        if drange < float(headers[idx][k]['minRange']):
                            headers[idx][k]['dmax'] = headers[idx][k]['dmin'] + float(headers[idx][k]['minRange'])

        # Collect settings for shared keys
        shared_keys = defaultdict(lambda: dict())
        for idx, hs in enumerate(headers):
            for k in hs.keys():
                sk = headers[idx][k]['shared_key']
                if sk is not None:
                    shared_keys[sk]['dmax']  = max(headers[idx][k]['dmax'], shared_keys[sk].get('dmax', headers[idx][k]['dmax']))
                    shared_keys[sk]['dmin']  = max(headers[idx][k]['dmin'], shared_keys[sk].get('dmin', headers[idx][k]['dmin']))

        # Overwrite shared key settings and at the same time assign to buckets for sorting
        # Within each section of headers, sort explicitly by 'title' if the dict
        # is not already ordered, so the final ordering is by:
        # placement > section > explicit_ordering > title
        # Of course, the user can shuffle these manually.
        self.headers_in_order = defaultdict(list)

        for idx, hs in enumerate(headers):
            keys_in_section = hs.keys()
            if type(hs) is not OrderedDict:
                keys_in_section = sorted(keys_in_section, key=lambda k: headers[idx][k]['title'])

            for k in keys_in_section:
                sk = headers[idx][k]['shared_key']
                if sk is not None:
                    headers[idx][k]['dmax'] = shared_keys[sk]['dmax']
                    headers[idx][k]['dmin'] = shared_keys[sk]['dmin']

                self.headers_in_order[headers[idx][k]['placement']].append((idx, k))

        # Skip any data that is not used in the table
        # Would be ignored for making the table anyway, but can affect whether a beeswarm plot is used
        for idx, d in enumerate(data):
            for s_name in list(d.keys()):
                if not any ( h in data[idx][s_name].keys() for h in headers[idx]):
                    del(data[idx][s_name])

        # Assign to class
        self.data = data
        self.headers = headers
        self.pconfig = pconfig

    def get_headers_in_order(self):
        """Gets the headers in the order they want to be displayed.
           Returns a list of triplets: (idx, key, header_info)
        """
        res = list()
        # Scan through self.headers_in_order and just bolt on the actual header info
        for bucket in sorted(self.headers_in_order):
            for idx, k in self.headers_in_order[bucket]:
                res.append( (idx, k, self.headers[idx][k]) )
        return res



#!/usr/bin/env python

""" MultiQC module to parse output from Cutadapt """

from __future__ import print_function
from collections import OrderedDict
from distutils.version import StrictVersion
import logging
import re

from multiqc.plots import bargraph, linegraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """
    Cutadapt module class, parses stderr logs.
    Also understands logs saved by Trim Galore!
    (which contain cutadapt logs)
    """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='Cutadapt', anchor='cutadapt',
        href='https://cutadapt.readthedocs.io/',
        info="is a tool to find and remove adapter sequences, primers, poly-A"\
         "tails and other types of unwanted sequence from your high-throughput"\
         " sequencing reads.")

        # Find and load any Cutadapt reports
        self.cutadapt_data = dict()
        self.cutadapt_length_counts = dict()
        self.cutadapt_length_exp = dict()
        self.cutadapt_length_obsexp = dict()

        for f in self.find_log_files('cutadapt', filehandles=True):
            self.parse_cutadapt_logs(f)

        # Filter to strip out ignored sample names
        self.cutadapt_data = self.ignore_samples(self.cutadapt_data)

        if len(self.cutadapt_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.cutadapt_data)))

        # Write parsed report data to a file
        self.write_data_file(self.cutadapt_data, 'multiqc_cutadapt')

        # Basic Stats Table
        self.cutadapt_general_stats_table()

        # Bar plot with number of reads trimmed
        self.cutadapt_filtered_barplot()

        # Trimming Length Profiles
        self.cutadapt_length_trimmed_plot()


    def parse_cutadapt_logs(self, f):
        """ Go through log file looking for cutadapt output """
        fh = f['f']
        regexes = {
            '1.7': {
                'bp_processed': "Total basepairs processed:\s*([\d,]+) bp",
                'bp_written': "Total written \(filtered\):\s*([\d,]+) bp",
                'quality_trimmed': "Quality-trimmed:\s*([\d,]+) bp",
                'r_processed': "Total reads processed:\s*([\d,]+)",
                'pairs_processed': "Total read pairs processed:\s*([\d,]+)",
                'r_with_adapters': "Reads with adapters:\s*([\d,]+)",
                'r1_with_adapters': "Read 1 with adapter:\s*([\d,]+)",
                'r2_with_adapters': "Read 2 with adapter:\s*([\d,]+)",
                'r_too_short': "Reads that were too short:\s*([\d,]+)",
                'pairs_too_short': "Pairs that were too short:\s*([\d,]+)",
                'r_too_long': "Reads that were too long:\s*([\d,]+)",
                'pairs_too_long': "Pairs that were too long:\s*([\d,]+)",
                'r_too_many_N': "Reads with too many N:\s*([\d,]+)",
                'pairs_too_many_N': "Pairs with too many N:\s*([\d,]+)",
                'r_written': "Reads written \(passing filters\):\s*([\d,]+)",
                'pairs_written': "Pairs written \(passing filters\):\s*([\d,]+)",
            },
            '1.6': {
                'r_processed': "Processed reads:\s*([\d,]+)",
                'bp_processed': "Processed bases:\s*([\d,]+) bp",
                'r_trimmed': "Trimmed reads:\s*([\d,]+)",
                'quality_trimmed': "Quality-trimmed:\s*([\d,]+) bp",
                'bp_trimmed': "Trimmed bases:\s*([\d,]+) bp",
                'too_short': "Too short reads:\s*([\d,]+)",
                'too_long': "Too long reads:\s*([\d,]+)",
            }
        }
        s_name = None
        cutadapt_version = '1.7'
        log_section = None
        for l in fh:
            # New log starting
            if 'cutadapt' in l:
                s_name = None
                c_version = re.match(r'This is cutadapt ([\d\.]+)', l)
                if c_version:
                    try:
                        assert(StrictVersion(c_version.group(1)) <= StrictVersion('1.6'))
                        cutadapt_version = '1.6'
                    except:
                        cutadapt_version = '1.7'
                c_version_old = re.match(r'cutadapt version ([\d\.]+)', l)
                if c_version_old:
                    try:
                        assert(StrictVersion(c_version.group(1)) <= StrictVersion('1.6'))
                        cutadapt_version = '1.6'
                    except:
                        # I think the pattern "cutadapt version XX" is only pre-1.6?
                        cutadapt_version = '1.6'
            # Get sample name from end of command line params
            if l.startswith('Command line parameters'):
                s_name = l.split()[-1]
                # Manage case where sample name is '-' (reading from stdin)
                if s_name == '-':
                    s_name = f['s_name']
                else:
                    s_name = self.clean_s_name(s_name, f['root'])
                if s_name in self.cutadapt_data:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
                self.cutadapt_data[s_name] = dict()

            if s_name is not None:
                self.add_data_source(f, s_name)

                # Search regexes for overview stats
                for k, r in regexes[cutadapt_version].items():
                    match = re.search(r, l)
                    if match:
                        self.cutadapt_data[s_name][k] = int(match.group(1).replace(',', ''))

                # Starting a new section
                if '===' in l:
                    log_section = l.strip().strip('=').strip()

                # Histogram showing lengths trimmed
                if 'length' in l and 'count' in l and 'expect' in l:
                    plot_sname = s_name
                    if log_section is not None:
                        plot_sname = '{} - {}'.format(s_name, log_section)
                    self.cutadapt_length_counts[plot_sname] = dict()
                    self.cutadapt_length_exp[plot_sname] = dict()
                    self.cutadapt_length_obsexp[plot_sname] = dict()
                    # Nested loop to read this section while the regex matches
                    for l in fh:
                        r_seqs = re.search("^(\d+)\s+(\d+)\s+([\d\.]+)", l)
                        if r_seqs:
                            a_len = int(r_seqs.group(1))
                            self.cutadapt_length_counts[plot_sname][a_len] = int(r_seqs.group(2))
                            self.cutadapt_length_exp[plot_sname][a_len] = float(r_seqs.group(3))
                            if float(r_seqs.group(3)) > 0:
                                self.cutadapt_length_obsexp[plot_sname][a_len] = float(r_seqs.group(2)) / float(r_seqs.group(3))
                            else:
                                # Cheating, I know. Infinity is difficult to plot.
                                self.cutadapt_length_obsexp[plot_sname][a_len] = float(r_seqs.group(2))
                        else:
                            break

        # Calculate a few extra numbers of our own
        for s_name, d in self.cutadapt_data.items():
            # Percent trimmed
            if 'bp_processed' in d and 'bp_written' in d:
                self.cutadapt_data[s_name]['percent_trimmed'] = (float(d['bp_processed'] - d['bp_written']) / d['bp_processed']) * 100
            elif 'bp_processed' in d and 'bp_trimmed' in d:
                self.cutadapt_data[s_name]['percent_trimmed'] = ((float(d.get('bp_trimmed', 0)) + float(d.get('quality_trimmed', 0))) / d['bp_processed']) * 100

    def cutadapt_general_stats_table(self):
        """ Take the parsed stats from the Cutadapt report and add it to the
        basic stats table at the top of the report """

        headers = {}
        headers['percent_trimmed'] = {
            'title': '% BP Trimmed',
            'description': '% Total Base Pairs trimmed',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'RdYlBu-rev'
        }
        self.general_stats_addcols(self.cutadapt_data, headers)

    def cutadapt_filtered_barplot(self):
        """ Bar plot showing proportion of reads trimmed """

        pconfig = {
            'id': 'cutadapt_filtered_reads_plot',
            'title': 'Cutadapt: Filtered Reads',
            'ylab': 'Counts'
        }

        # We just use all categories. If a report is generated with a mixture
        # of SE and PE data then this means quite a lot of categories.
        # Usually, only a single data type is used though - in that case
        # any categories with 0 across all samples will be ignored.
        cats = OrderedDict()
        cats['pairs_written'] = { 'name': 'Pairs passing filters' }
        cats['r_written'] = { 'name': 'Reads passing filters' }
        cats['pairs_too_short'] = { 'name': 'Pairs that were too short' }
        cats['r_too_short'] = { 'name': 'Reads that were too short' }
        cats['pairs_too_long'] = { 'name': 'Pairs that were too long' }
        cats['r_too_long'] = { 'name': 'Reads that were too long' }
        cats['pairs_too_many_N'] = { 'name': 'Pairs with too many N' }
        cats['r_too_many_N'] = { 'name': 'Reads with too many N' }

        self.add_section(
            name = 'Filtered Reads',
            anchor = 'cutadapt_filtered_reads',
            description = 'This plot shows the number of reads (SE) / pairs (PE) removed by Cutadapt.',
            plot = bargraph.plot(self.cutadapt_data, cats, pconfig)
        )

    def cutadapt_length_trimmed_plot (self):
        """ Generate the trimming length plot """

        pconfig = {
            'id': 'cutadapt_trimmed_sequences_plot',
            'title': 'Cutadapt: Lengths of Trimmed Sequences',
            'ylab': 'Counts',
            'xlab': 'Length Trimmed (bp)',
            'xDecimals': False,
            'ymin': 0,
            'tt_label': '<b>{point.x} bp trimmed</b>: {point.y:.0f}',
            'data_labels': [
                {'name': 'Counts', 'ylab': 'Count'},
                {'name': 'Obs/Exp', 'ylab': 'Observed / Expected'}
            ]
        }

        self.add_section(
            name = 'Trimmed Sequence Lengths',
            anchor = 'cutadapt_trimmed_sequences',
            description = 'This plot shows the number of reads with certain lengths of adapter trimmed.',
            helptext = '''
            Obs/Exp shows the raw counts divided by the number expected due to sequencing errors.
            A defined peak may be related to adapter length.

            See the [cutadapt documentation](http://cutadapt.readthedocs.org/en/latest/guide.html#how-to-read-the-report)
            for more information on how these numbers are generated.
            ''',
            plot = linegraph.plot([self.cutadapt_length_counts, self.cutadapt_length_obsexp], pconfig)
        )



#!/usr/bin/env python

""" MultiQC module to parse output from HTSeq Count """

from __future__ import print_function
from collections import OrderedDict
import logging

from multiqc import config
from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='HTSeq Count',
        anchor='htseq', target='HTSeq Count',
        href='http://www-huber.embl.de/HTSeq/doc/count.html',
        info=" is part of the HTSeq Python package - it takes a file with aligned sequencing "\
             "reads, plus a list of genomic features and counts how many reads map to each feature.")

        # Find and load any HTSeq Count reports
        self.htseq_data = dict()
        self.htseq_keys = list()
        for f in self.find_log_files('htseq', filehandles=True):
            parsed_data = self.parse_htseq_report(f)
            if parsed_data is not None:
                self.htseq_data[f['s_name']] = parsed_data

        # Filter to strip out ignored sample names
        self.htseq_data = self.ignore_samples(self.htseq_data)

        if len(self.htseq_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.htseq_data)))

        # Write parsed report data to a file
        self.write_data_file(self.htseq_data, 'multiqc_htseq')

        # Basic Stats Table
        self.htseq_stats_table()

        # Assignment bar plot
        self.add_section( plot = self.htseq_counts_chart() )


    def parse_htseq_report (self, f):
        """ Parse the HTSeq Count log file. """
        keys = [ '__no_feature', '__ambiguous', '__too_low_aQual', '__not_aligned', '__alignment_not_unique' ]
        parsed_data = dict()
        assigned_counts = 0
        for l in f['f']:
            s = l.split("\t")
            if s[0] in keys:
                parsed_data[s[0][2:]] = int(s[-1])
            else:
                try:
                    assigned_counts += int(s[-1])
                except (ValueError, IndexError):
                    pass
        if len(parsed_data) > 0:
            parsed_data['assigned'] = assigned_counts
            parsed_data['total_count'] = sum([v for v in parsed_data.values()])
            try:
                parsed_data['percent_assigned'] = (float(parsed_data['assigned']) / float(parsed_data['total_count'])) * 100.0
            except ZeroDivisionError:
                parsed_data['percent_assigned'] = 0
            return parsed_data
        return None


    def htseq_stats_table(self):
        """ Take the parsed stats from the HTSeq Count report and add them to the
        basic stats table at the top of the report """

        headers = OrderedDict()
        headers['percent_assigned'] = {
            'title': '% Assigned',
            'description': '% Assigned reads',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'RdYlGn'
        }
        headers['assigned'] = {
            'title': '{} Assigned'.format(config.read_count_prefix),
            'description': 'Assigned Reads ({})'.format(config.read_count_desc),
            'min': 0,
            'scale': 'PuBu',
            'modify': lambda x: float(x) * config.read_count_multiplier,
            'shared_key': 'read_count'
        }
        self.general_stats_addcols(self.htseq_data, headers)


    def htseq_counts_chart (self):
        """ Make the HTSeq Count assignment rates plot """
        cats = OrderedDict()
        cats['assigned'] =      { 'name': 'Assigned' }
        cats['ambiguous'] =     { 'name': 'Ambiguous' }
        cats['alignment_not_unique'] = { 'name': 'Alignment Not Unique' }
        cats['no_feature'] =    { 'name': 'No Feature' }
        cats['too_low_aQual'] = { 'name': 'Too Low aQual' }
        cats['not_aligned'] =   { 'name': 'Not Aligned' }
        config = {
            'id': 'htseq_assignment_plot',
            'title': 'HTSeq: Count Assignments',
            'ylab': '# Reads',
            'hide_zero_cats': False,
            'cpswitch_counts_label': 'Number of Reads'
        }
        return bargraph.plot(self.htseq_data, cats, config)



#!/usr/bin/env python
""" MultiQC module to parse output from sickle """

from collections import OrderedDict
import logging
import re

from multiqc import config
from multiqc.modules.base_module import BaseMultiqcModule
from multiqc.plots import bargraph

log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    def __init__(self):
        super(MultiqcModule, self).__init__(
            name = 'Sickle',
            anchor = 'sickle',
            href = "https://github.com/najoshi/sickle",
            info = "A windowed adaptive trimming tool for FASTQ files using quality."
        )

        # parse list of log files
        self.sickle_data = dict()
        for f in self.find_log_files('sickle'):
            parsed_data = self.parse_logs(f['f'])
            if len(parsed_data):
                if f['s_name'] in self.sickle_data:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
                self.sickle_data[f['s_name']] = parsed_data
                self.add_data_source(f)

        self.sickle_data = self.ignore_samples(self.sickle_data)

        # no file found
        if len(self.sickle_data) == 0:
            raise UserWarning

        self.write_data_file(self.sickle_data, 'multiqc_sickle')

        self.sickle_general_stats_table()
        self.read_count_plot()

    def parse_logs(self, f):
        """ Parse the Sickle standard output """
        regexes = [
            # Paired-end
            ['reads_paired_kept', re.compile("FastQ paired records kept: ([\d,]+) .*")],
            ['reads_single_kept', re.compile("FastQ single records kept: ([\d,]+).*")],
            ['reads_paired_discarded', re.compile("FastQ paired records discarded: ([\d,]+) .*")],
            ['reads_single_discarded', re.compile("FastQ single records discarded: ([\d,]+) .*")],
            # Single-end
            ['reads_single_kept', re.compile("FastQ records kept: ([\d,]+)")],
            ['reads_single_discarded', re.compile("FastQ records discarded: ([\d,]+)")],
        ]
        data = {}
        for l in f.splitlines():
            s = l.split(':')
            # Search regexes for overview stats# Search regexes for overview stats
            for k, regex in regexes:
                match = regex.search(l)
                if match:
                    data[k] = int(match.group(1).replace(',', ''))

        # if paired data : compute total for general stats table
        if "reads_single_kept" in data:
            data["reads_total_kept"] = data.get("reads_paired_kept", 0) + data["reads_single_kept"]

        if "reads_single_discarded" in data:
            data["reads_total_discarded"] = data.get("reads_paired_discarded", 0) + data["reads_single_discarded"]

        # Calculate percentage discarded
        if 'reads_total_kept' in data and 'reads_single_discarded' in data:
            data['percentage_discarded'] = (float(data["reads_total_discarded"]) / float(data["reads_total_kept"] + data["reads_total_discarded"])) * 100.0

        return data

    def sickle_general_stats_table(self):
        """ Take the parsed stats from the sickle report and add
        number of kept reads into the generam stats table """

        headers = {}
        headers['percentage_discarded'] = {
            'title': '% Reads discarded',
            'description': 'Percentage of reads discarded',
            'min': 0,
            'max': 100,
            'suffix': '%',
            'scale': 'OrRd'
        }
        headers['reads_total_kept'] = {
            'title': '{} Reads kept'.format(config.read_count_prefix),
            'description': 'Number of reads kept ({})'.format(config.read_count_desc),
            'modify': lambda x: x * config.read_count_multiplier,
            'scale': 'BuGn'
        }
        self.general_stats_addcols(self.sickle_data, headers)

    def read_count_plot (self):
        """ Stacked bar plot showing counts of reads """
        pconfig = {
            'id': 'sickle_sequence_counts_plot',
            'title': 'Sickle: Sequence Counts',
            'ylab': 'Number of reads',
            'cpswitch_counts_label': 'Number of reads'
        }
        pcats = OrderedDict()
        pcats['reads_paired_kept'] = {'name': 'Paired reads kept', 'color': '#1f78b4'}
        pcats['reads_single_kept'] = {'name': 'Single reads kept', 'color': '#a6cee3'}
        pcats['reads_paired_discarded'] = {'name': 'Paired reads discarded', 'color': '#ff7f00'}
        pcats['reads_single_discarded'] = {'name': 'Single reads discarded', 'color': '#fdae61'}

        self.add_section (
            name = 'Sequence counts',
            anchor = 'sickle_sequence_counts',
            description = 'Number of reads discarded by Sickle.',
            plot = bargraph.plot(self.sickle_data, pcats, pconfig)
        )



#!/usr/bin/env python

""" MultiQC module to parse output from Samtools rmdup """

import logging
import re
from collections import OrderedDict
from multiqc.plots import bargraph

# Initialise the logger
log = logging.getLogger(__name__)


class RmdupReportMixin():

    def parse_samtools_rmdup(self):
        """ Find Samtools rmdup logs and parse their data """

        self.samtools_rmdup = dict()
        for f in self.find_log_files('samtools/rmdup', filehandles=True):
            # Example below:
            # [bam_rmdupse_core] 26602816 / 103563641 = 0.2569 in library '   '
            dups_regex = "\[bam_rmdups?e?_core\] (\d+) / (\d+) = (\d+\.\d+) in library '(.*)'"
            s_name = f['s_name']
            for l in f['f']:
                match = re.search(dups_regex, l)
                if match:
                    library_name = match.group(4).strip()
                    if library_name != '':
                        s_name = library_name
                    if s_name in self.samtools_rmdup:
                        log.debug("Duplicate sample name found in {}! Overwriting: {}".format(f['fn'], s_name))
                    self.add_data_source(f, s_name)
                    self.samtools_rmdup[s_name] = dict()
                    self.samtools_rmdup[s_name]['n_dups'] = int(match.group(1))
                    self.samtools_rmdup[s_name]['n_tot'] = int(match.group(2))
                    self.samtools_rmdup[s_name]['n_unique'] = int(match.group(2)) - int(match.group(1))
                    self.samtools_rmdup[s_name]['pct_dups'] = float(match.group(3))*100

        # Filter to strip out ignored sample names
        self.samtools_rmdup = self.ignore_samples(self.samtools_rmdup)

        if len(self.samtools_rmdup) > 0:
            # Write parsed report data to a file
            self.write_data_file(self.samtools_rmdup, 'multiqc_samtools_rmdup')

            # Make a bar plot showing duplicates
            keys = OrderedDict()
            keys['n_unique'] = {'name': 'Non-duplicated reads'}
            keys['n_dups'] = {'name': 'Duplicated reads'}
            pconfig = {
                'id': 'samtools_rmdup_plot',
                'title': 'Samtools rmdup: Duplicate alignments',
                'ylab': 'Number of reads',
                'yDecimals': False
            }
            self.add_section (
                name = 'Duplicates removed',
                anchor = 'samtools-rmdup',
                plot = bargraph.plot(self.samtools_rmdup, keys, pconfig)
            )

            # Add a column to the General Stats table
            # General Stats Table
            stats_headers = OrderedDict()
            stats_headers['pct_dups'] = {
                'title': '% Dups',
                'description': 'Percent of duplicate alignments',
                'min': 0,
                'max': 100,
                'suffix': '%',
                'scale': 'OrRd'
            }
            self.general_stats_addcols(self.samtools_rmdup, stats_headers)

        return len(self.samtools_rmdup)



#!/usr/bin/env python

""" MultiQC submodule to parse output from RSeQC bam_stat.py
http://rseqc.sourceforge.net/#bam-stat-py """

from collections import OrderedDict
import logging
import re

from multiqc.plots import beeswarm

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """ Find RSeQC bam_stat reports and parse their data """

    # Set up vars
    self.bam_stat_data = dict()
    regexes = {
        'total_records': r"Total records:\s*(\d+)",
        'qc_failed': r"QC failed:\s*(\d+)",
        'optical_pcr_duplicate': r"Optical/PCR duplicate:\s*(\d+)",
        'non_primary_hits': r"Non primary hits\s*(\d+)",
        'unmapped_reads': r"Unmapped reads:\s*(\d+)",
        'mapq_lt_mapq_cut_non-unique': r"mapq < mapq_cut \(non-unique\):\s*(\d+)",
        'mapq_gte_mapq_cut_unique': r"mapq >= mapq_cut \(unique\):\s*(\d+)",
        'read_1': r"Read-1:\s*(\d+)",
        'read_2': r"Read-2:\s*(\d+)",
        'reads_map_to_sense': r"Reads map to '\+':\s*(\d+)",
        'reads_map_to_antisense': r"Reads map to '-':\s*(\d+)",
        'non-splice_reads': r"Non-splice reads:\s*(\d+)",
        'splice_reads': r"Splice reads:\s*(\d+)",
        'reads_mapped_in_proper_pairs': r"Reads mapped in proper pairs:\s*(\d+)",
        'proper-paired_reads_map_to_different_chrom': r"Proper-paired reads map to different chrom:\s*(\d+)",
    }

    #intiate PE check
    is_paired_end=False

    # Go through files and parse data using regexes
    for f in self.find_log_files('rseqc/bam_stat'):
        d = dict()
        for k, r in regexes.items():
            r_search = re.search(r, f['f'], re.MULTILINE)
            if r_search:
                d[k] = int(r_search.group(1))

        # Calculate some percentages
        if 'total_records' in d:
            t = float(d['total_records'])
            if 'mapq_gte_mapq_cut_unique' in d:
                d['unique_percent'] = (float(d['mapq_gte_mapq_cut_unique']) / t)*100.0
            if 'reads_mapped_in_proper_pairs' in d:
                d['proper_pairs_percent'] = (float(d['reads_mapped_in_proper_pairs']) / t)*100.0

        if len(d) > 0:
            if f['s_name'] in self.bam_stat_data:
                log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
            self.add_data_source(f, section='bam_stat')
            #Check if SE or PE
            if d['read_2'] != 0:
                is_paired_end = True
            self.bam_stat_data[f['s_name']] = d

    # Filter to strip out ignored sample names
    self.bam_stat_data = self.ignore_samples(self.bam_stat_data)

    if len(self.bam_stat_data) > 0:
        # Write to file
        self.write_data_file(self.bam_stat_data, 'multiqc_rseqc_bam_stat')

        # Add to general stats table
        self.general_stats_headers['proper_pairs_percent'] = {
            'title': '% Proper Pairs',
            'description': '% Reads mapped in proper pairs',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'RdYlGn'
        }
        for s_name in self.bam_stat_data:
            if s_name not in self.general_stats_data:
                self.general_stats_data[s_name] = dict()

                #Only write if PE, i.e. there is something to write
                if is_paired_end:
                    self.general_stats_data[s_name].update( self.bam_stat_data[s_name] )

        # Make dot plot of counts
        pconfig = {
            'id': 'rseqc_bam_stat'
        }
        keys = OrderedDict()
        defaults = {
            'min': 0,
            'shared_key': 'read_count',
            'decimalPlaces': 2,
            'modify': lambda x: float(x) / 1000000.0,
        }
        keys['total_records'] = dict(defaults, **{'title': 'Total records' })
        keys['qc_failed'] = dict(defaults, **{'title': 'QC failed' })
        keys['optical_pcr_duplicate'] = dict(defaults, **{'title': 'Duplicates', 'description': 'Optical/PCR duplicate' })
        keys['non_primary_hits'] = dict(defaults, **{'title': 'Non primary hit' })
        keys['unmapped_reads'] = dict(defaults, **{'title': 'Unmapped', 'description': 'Unmapped reads' })
        keys['mapq_lt_mapq_cut_non'] = dict(defaults, **{'title': 'Non-unique', 'description': 'mapq < mapq_cut (non-unique)' })
        keys['mapq_gte_mapq_cut_unique'] = dict(defaults, **{'title': 'Unique', 'description': 'mapq >= mapq_cut (unique)' })
        if is_paired_end:
            keys['read_1'] = dict(defaults, **{'title': 'Read-1' })
            keys['read_2'] = dict(defaults, **{'title': 'Read-2' })
        keys['reads_map_to_sense'] = dict(defaults, **{'title': '+ve strand', 'description': "Reads map to '+'" })
        keys['reads_map_to_antisense'] = dict(defaults, **{'title': '-ve strand', 'description': "Reads map to '-'" })
        keys['non-splice_reads'] = dict(defaults, **{'title': 'Non-splice reads' })
        keys['splice_reads'] = dict(defaults, **{'title': 'Splice reads' })
        if is_paired_end:
            keys['reads_mapped_in_proper_pairs'] = dict(defaults, **{
                        'title': 'Proper pairs', 'description':'Reads mapped in proper pairs' })
            keys['proper-paired_reads_map_to_different_chrom'] = dict(defaults, **{
                        'title': 'Different chrom', 'description': 'Proper-paired reads map to different chrom' })

        self.add_section (
            name = 'Bam Stat',
            anchor = 'rseqc-bam_stat',
            description = 'All numbers reported in millions.',
            plot = beeswarm.plot(self.bam_stat_data, keys, pconfig)
        )

    # Return number of samples found
    return len(self.bam_stat_data)



#!/usr/bin/env python

""" MultiQC module to parse output from Picard """

from __future__ import print_function
from collections import OrderedDict
import logging

from multiqc.modules.base_module import BaseMultiqcModule

# Import the Picard submodules
from . import AlignmentSummaryMetrics
from . import BaseDistributionByCycleMetrics
from . import GcBiasMetrics
from . import HsMetrics
from . import InsertSizeMetrics
from . import MarkDuplicates
from . import OxoGMetrics
from . import QualityByCycleMetrics
from . import QualityScoreDistributionMetrics
from . import QualityYieldMetrics
from . import RnaSeqMetrics
from . import RrbsSummaryMetrics
from . import TargetedPcrMetrics
from . import VariantCallingMetrics
from . import ValidateSamFile
from . import WgsMetrics


# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """ Picard is a collection of scripts. This MultiQC module
    supports some but not all. The code for each script is split
    into its own file and adds a section to the module output if
    logs are found."""

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='Picard', anchor='picard',
        href='http://broadinstitute.github.io/picard/',
        info="is a set of Java command line tools for manipulating high-"\
        "throughput sequencing data.")

        # Set up class objects to hold parsed data
        self.general_stats_headers = OrderedDict()
        self.general_stats_data = dict()
        n = dict()

        # Call submodule functions
        n['AlignmentMetrics'] = AlignmentSummaryMetrics.parse_reports(self)
        if n['AlignmentMetrics'] > 0:
            log.info("Found {} AlignmentSummaryMetrics reports".format(n['AlignmentMetrics']))

        n['BaseDistributionByCycleMetrics'] = BaseDistributionByCycleMetrics.parse_reports(self)
        if n['BaseDistributionByCycleMetrics'] > 0:
            log.info("Found {} BaseDistributionByCycleMetrics reports".format(n['BaseDistributionByCycleMetrics']))

        n['GcBiasMetrics'] = GcBiasMetrics.parse_reports(self)
        if n['GcBiasMetrics'] > 0:
            log.info("Found {} GcBiasMetrics reports".format(n['GcBiasMetrics']))

        n['HsMetrics'] = HsMetrics.parse_reports(self)
        if n['HsMetrics'] > 0:
            log.info("Found {} HsMetrics reports".format(n['HsMetrics']))

        n['InsertSizeMetrics'] = InsertSizeMetrics.parse_reports(self)
        if n['InsertSizeMetrics'] > 0:
            log.info("Found {} InsertSizeMetrics reports".format(n['InsertSizeMetrics']))

        n['MarkDuplicates'] = MarkDuplicates.parse_reports(self)
        if n['MarkDuplicates'] > 0:
            log.info("Found {} MarkDuplicates reports".format(n['MarkDuplicates']))

        n['OxoGMetrics'] = OxoGMetrics.parse_reports(self)
        if n['OxoGMetrics'] > 0:
            log.info("Found {} OxoGMetrics reports".format(n['OxoGMetrics']))
        
        n['QualityByCycleMetrics'] = QualityByCycleMetrics.parse_reports(self)
        if n['QualityByCycleMetrics'] > 0:
            log.info("Found {} QualityByCycleMetrics reports".format(n['QualityByCycleMetrics']))
        
        n['QualityScoreDistributionMetrics'] = QualityScoreDistributionMetrics.parse_reports(self)
        if n['QualityScoreDistributionMetrics'] > 0:
            log.info("Found {} QualityScoreDistributionMetrics reports".format(n['QualityScoreDistributionMetrics']))
        
        n['QualityYieldMetrics'] = QualityYieldMetrics.parse_reports(self)
        if n['QualityYieldMetrics'] > 0:
            log.info("Found {} QualityYieldMetrics reports".format(n['QualityYieldMetrics']))

        n['RnaSeqMetrics'] = RnaSeqMetrics.parse_reports(self)
        if n['RnaSeqMetrics'] > 0:
            log.info("Found {} RnaSeqMetrics reports".format(n['RnaSeqMetrics']))

        n['RrbsSummaryMetrics'] = RrbsSummaryMetrics.parse_reports(self)
        if n['RrbsSummaryMetrics'] > 0:
            log.info("Found {} RrbsSummaryMetrics reports".format(n['RrbsSummaryMetrics']))

        n['TargetedPcrMetrics'] = TargetedPcrMetrics.parse_reports(self)
        if n['TargetedPcrMetrics'] > 0:
            log.info("Found {} TargetedPcrMetrics reports".format(n['TargetedPcrMetrics']))

        n['VariantCallingMetrics'] = VariantCallingMetrics.parse_reports(self)
        if n['VariantCallingMetrics'] > 0:
            log.info("Found {} VariantCallingMetrics reports".format(n['VariantCallingMetrics']))

        n['ValidateSamFile'] = ValidateSamFile.parse_reports(self)
        if n['ValidateSamFile'] > 0:
            log.info("Found {} ValidateSamFile reports".format(n['ValidateSamFile']))

        n['WgsMetrics'] = WgsMetrics.parse_reports(self)
        if n['WgsMetrics'] > 0:
            log.info("Found {} WgsMetrics reports".format(n['WgsMetrics']))

        # Exit if we didn't find anything
        if sum(n.values()) == 0:
            raise UserWarning

        # Add to the General Stats table (has to be called once per MultiQC module)
        self.general_stats_addcols(self.general_stats_data, self.general_stats_headers)

    # Helper functions
    def multiply_hundred(self, val):
        try:
            val = float(val) * 100
        except ValueError:
            pass
        return val




#!/usr/bin/env python

""" MultiQC submodule to parse output from Picard TargetedPcrMetrics """

from collections import OrderedDict
import logging
import os
import re

from multiqc.plots import bargraph

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """ Find Picard TargetedPcrMetrics reports and parse their data """

    # Set up vars
    self.picard_pcrmetrics_data = dict()
    self.picard_pcrmetrics_samplestats = dict()

    # Go through logs and find Metrics
    for f in self.find_log_files('picard/pcr_metrics', filehandles=True):
        s_name = None
        for l in f['f']:
            # New log starting
            if 'TargetedPcrMetrics' in l and 'INPUT' in l:
                s_name = None
                # Pull sample name from input
                fn_search = re.search(r"INPUT(?:=|\s+)(\[?[^\s]+\]?)", l, flags=re.IGNORECASE)
                if fn_search:
                    s_name = os.path.basename(fn_search.group(1).strip('[]'))
                    s_name = self.clean_s_name(s_name, f['root'])

            if s_name is not None:
                if 'TargetedPcrMetrics' in l and '## METRICS CLASS' in l:
                    keys = f['f'].readline().strip("\n").split("\t")
                    vals = f['f'].readline().strip("\n").split("\t")
                    if len(vals) == len(keys):
                        if s_name in self.picard_pcrmetrics_data:
                            log.debug("Duplicate sample name found in {}! Overwriting: {}".format(f['fn'], s_name))
                        self.add_data_source(f, s_name, section='TargetedPcrMetrics')
                        self.picard_pcrmetrics_data[s_name] = dict()
                        for i, k in enumerate(keys):
                            try:
                                # Multiply percentages by 100
                                if k.startswith('PCT_'):
                                    vals[i] = float(vals[i]) * 100.0
                                self.picard_pcrmetrics_data[s_name][k] = float(vals[i])
                            except ValueError:
                                self.picard_pcrmetrics_data[s_name][k] = vals[i]

    # Filter to strip out ignored sample names
    self.picard_pcrmetrics_data = self.ignore_samples(self.picard_pcrmetrics_data)

    if len(self.picard_pcrmetrics_data) > 0:

        # Write parsed data to a file
        self.write_data_file(self.picard_pcrmetrics_data, 'multiqc_picard_pcrmetrics')

        # Add to general stats table
        self.general_stats_headers['PCT_AMPLIFIED_BASES'] = {
            'title': '% Amplified Bases',
            'description': 'The fraction of aligned bases that mapped to or near an amplicon.',
            'min': 0,
            'max': 100,
            'suffix': '%',
            'scale': 'BrBG'
        }
        self.general_stats_headers['MEDIAN_TARGET_COVERAGE'] = {
            'title': 'Median Target Coverage',
            'description': 'The median coverage of reads that mapped to target regions of an experiment.',
            'min': 0,
            'suffix': 'X',
            'scale': 'GnBu',
        }

        for s_name in self.picard_pcrmetrics_data:
            if s_name not in self.general_stats_data:
                self.general_stats_data[s_name] = dict()
            self.general_stats_data[s_name].update( self.picard_pcrmetrics_data[s_name] )

        # Bar plot of ignored bases
        keys = OrderedDict()
        keys['ON_AMPLICON_BASES'] = {'name': 'On-amplicon bases'}
        keys['NEAR_AMPLICON_BASES'] = {'name': 'Near-amplicon bases'}
        keys['OFF_AMPLICON_BASES'] = {'name': 'Off-amplicon bases', 'color': '#f28f43'}

        # Config for the plot
        pconfig = {
            'id': 'picard_pcr_metrics_bases',
            'title': 'Picard: PCR Amplicon Bases',
            'ylab': '# Bases',
            'cpswitch_counts_label': '# Bases',
            'hide_zero_cats': False
        }

        self.add_section (
            name = 'PCR Amplicon Bases',
            anchor = 'picard-pcrmetrics-bases',
            description = 'Metrics about reads obtained from targeted PCR experiments.',
            helptext = '''
            This plot shows the number of bases aligned on or near to amplified regions of the genome.

            * `ON_AMPLICON_BASES`: The number of `PF_BASES_ALIGNED` that mapped to an amplified region of the genome.
            * `NEAR_AMPLICON_BASES`: The number of `PF_BASES_ALIGNED` that mapped to within a fixed interval of an amplified region, but not on a baited region.
            * `OFF_AMPLICON_BASES`: The number of `PF_BASES_ALIGNED` that mapped neither on or near an amplicon.

            For more information see the [Picard documentation](https://broadinstitute.github.io/picard/picard-metric-definitions.html#TargetedPcrMetrics).''',
            plot = bargraph.plot(self.picard_pcrmetrics_data, keys, pconfig)
        )


    # Return the number of detected samples to the parent module
    return len(self.picard_pcrmetrics_data)




#!/usr/bin/env python
"""MultiQC module to parse the output from SNPsplit"""
from collections import OrderedDict
import logging
import re
import yaml

from multiqc import config
from multiqc.modules.base_module import BaseMultiqcModule
from multiqc.plots import bargraph

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    def __init__(self):
        super(MultiqcModule, self).__init__(
            name='SNPsplit',
            anchor='SNPsplit',
            target='SNPsplit',
            href='https://www.bioinformatics.babraham.ac.uk/projects/SNPsplit/',
            info='A tool to determine allele-specific alignments from high-throughput sequencing experiments that have been aligned to N-masked genomes'
        )

        self.snpsplit_data = dict()

        # Parse log files generated with newer versions of SNPsplit (YAML)
        for f in self.find_log_files('snpsplit/new', filehandles=True):
            parsed_log = self.parse_new_snpsplit_log(f)
            self._save_parsed(parsed_log, f)

        # Parse log files generated with older versions of SNPsplit
        for f in self.find_log_files('snpsplit/old'):
            parsed_log = self.parse_old_snpsplit_log(f)
            self._save_parsed(parsed_log, f)

        # Filter --ignore-samples
        self.snpsplit_data = self.ignore_samples(self.snpsplit_data)

        if len(self.snpsplit_data) == 0:
            raise UserWarning
        log.info("Found {} reports".format(len(self.snpsplit_data)))

        self.write_data_file(self.snpsplit_data, 'multiqc_snpsplit')

        self.add_general_stats()
        self.allele_tagging_section()
        self.allele_sorting_section()

    def _save_parsed(self, parsed, f):
        s_name = self.clean_s_name(parsed[0], f['root'])
        if s_name in self.snpsplit_data:
            log.debug("Replacing duplicate sample {}".format(s_name))
        self.snpsplit_data[s_name] = parsed[1]
        self.add_data_source(f, s_name=s_name)

    def parse_new_snpsplit_log(self, f):
        data = next(yaml.load_all(f['f'], Loader=yaml.SafeLoader))
        flat_data = {}
        for k in data:
            for sk in data[k]:
                key = sk
                for prefix in ['PE_', 'SE_', 'HiC_']:
                    if sk.startswith(prefix):
                        key = sk[len(prefix):]
                flat_key = '{}_{}'.format(k.lower(), key)
                flat_data[flat_key] = data[k][sk]
        input_fn = data['Meta']['infile']
        return [input_fn, flat_data]

    def parse_old_snpsplit_log(self, f):
        input_fn = None
        d = dict()

        for line in f['f'].splitlines():
            # Parse the sample name
            match = re.match(r"Input file:\W+'([^']+)'", line)
            if match:
                # Report can have two sections with two input files,
                # the second sorting the output from the tagging
                # Just take the first instance, which is the initial input file
                if input_fn is None:
                    input_fn = match.group(1)
                continue

            # Log format 1: XXX description
            regex_patterns = [
                # Allele-tagging report
                ['tagging_g1', r"(\d+) reads were specific for genome 1"],
                ['tagging_g2', r"(\d+) reads were specific for genome 2"],
                ['tagging_unassignable', r"(\d+) reads were unassignable"],
                ['tagging_bizarre', r"(\d+) contained conflicting allele-specific SNPs"],
                ['tagging_unaligned', r"Reads were unaligned and hence skipped: (\d+)"],
                ['tagging_CT_positions_skipped', r"(\d+) reads that were unassignable contained C>T SNPs"],
                # Allele-specific sorting report
                ['sorting_conflicting', r"Reads contained conflicting SNP information:\W+(\d+)"]
            ]
            for (k, regex) in regex_patterns:
                match = re.match(regex, line)
                if match:
                    d[k] = int(match.group(1))
                    break

            # Format 2: Decription: XXX
            sorting_patterns = [
                # Tagging meta
                ['tagging_SNP_annotation', "SNP annotation file"],
                ['tagging_SNPs_stored', "SNPs stored in total"],
                ['tagging_N_was_known_SNP', "N was present in the list of known SNPs"],
                ['tagging_N_not_known', "N was not present in the list of SNPs"],
                # Allele sorting
                ['sorting_genome1', "Reads were specific for genome 1"],
                ['sorting_genome2', "Reads were specific for genome 2"],
                ['sorting_unassignable', "Reads were unassignable"],
                ['sorting_conflicting', "Read pairs contained conflicting SNP information"],
                # Hi-C data
                ['sorting_genome1_G1_G1', "Read pairs were specific for genome 1 (G1/G1)"],
                ['sorting_genome2_G2_G2', "Read pairs were specific for genome 2 (G2/G2)"],
                ['sorting_unassignable_UA_UA', "Read pairs were unassignable (UA/UA):"],
                ['sorting_G1_UA_total', "Read pairs were a mix of G1 and UA"],
                ['sorting_G2_UA_total', "Read pairs were a mix of G2 and UA"],
                ['sorting_G1_G2_total', "Read pairs were a mix of G1 and G2"],
            ]
            for (k, pattern) in sorting_patterns:
                if line.startswith(pattern):
                    try:
                        d[k] = int(line.split("\t")[-1].split()[0])
                    except ValueError:
                        d[k] = line.split("\t")[-1].split()[0]
                    break

            if 'tagging_N_was_known_SNP' in d and 'tagging_N_not_known' in d:
                n_total = d['tagging_N_was_known_SNP'] + d['tagging_N_not_known']
                d['tagging_percent_N_was_known_SNP'] = (d['tagging_N_was_known_SNP'] / float(n_total)) * 100.0

        return [input_fn, d]

    def add_general_stats(self):
        """ Add some columns to the General Statistics table at the top of the report """
        headers = OrderedDict()
        headers['tagging_SNP_annotation'] = {
            'title': 'SNP annotation',
            'description': 'Annotation file used for differentiating genomes',
            'scale': False,
            'modify': lambda x: '<code>{}</code>'.format(x),
            'hidden': True
        }
        headers['tagging_percent_N_was_known_SNP'] = {
            'title': '% Ns known SNP',
            'description': 'Percentage of detected SNPs in the sample that were also present in the annotation',
            'scale': 'RdYlGn',
            'format': '{:,.2f}',
            'max': 100,
            'min': 0,
            'suffix': '%'
        }
        headers['tagging_SNPs_stored'] = {
            'title': 'SNPs stored',
            'description': 'Total number of SNPs used for the analysis',
            'scale': 'PrGn',
            'format': '{:,.0f}',
            'hidden': True
        }
        self.general_stats_addcols(self.snpsplit_data, headers)

    def allele_tagging_section(self):
        ''' Allele-tagging report '''
        cats = OrderedDict()
        cats['tagging_g1'] = {'name':'Genome 1'}
        cats['tagging_g2'] = {'name':'Genome 2'}
        cats['tagging_unassignable'] = {'name':'Not assigned'}
        cats['tagging_bizarre'] = {'name':'Conflicting SNPs'}
        cats['tagging_unaligned'] = {'name':'Unaligned reads'}
        cats['tagging_CT_positions_skipped'] = {'name': 'C->T SNP'}

        # Subtract C->T from unassignable
        plot_data = {}
        for s_name, ss_data in self.snpsplit_data.items():
            if 'tagging_unassignable' and 'tagging_CT_positions_skipped' in ss_data:
                ss_data['tagging_unassignable'] -= ss_data['tagging_CT_positions_skipped']
            plot_data[s_name] = ss_data

        pconfig = {
            'id': 'snpsplit-allele-tagging-plot',
            'title': "SNPsplit: Allele-tagging report",
            'ylab': "Reads",
            'cpswitch_counts_label': 'Reads'
        }

        self.add_section(
            name = "Allele-tagging report",
            description = "Per-sample metrics of how many reads were assigned to each genome.",
            helptext = """
                Allele tagging works on a per-read basis. The results may therefore differ considerably
                from the Allele-specific sorting results if the samples were paired-end or Hi-C samples.

                For single hybrid genomes, Genome 1-specific reads are specific for the reference sequence
                and Genome 2-specific reads are specific for the straing specified with `--strain`.

                For dual hybrid genomes, Genome 1-specific reads are specific for the strain specified
                with `--strain`, and Genome 2-specific reads are specific for the straing specified with `--strain2`.

                Bar graph categories are:

                * `Genome 1`: Reads assigned to Genome 1
                * `Genome 2`: Reads assigned to Genome 2
                * `Not assigned`: Reads don't overlap a SNP
                * `Conflicting SNPs`: Reads contained allele-specific information for both alleles within the same read
                * `Unaligned reads`: Reads aren't aligned
                * `No match`: Reads overlap informative SNPs, but don't contain the expected nucleotide for either genome
                * `C->T SNP`: (Bisulfite sequencing data only) Read SNPs involved some form of C->T transition, rendering it non-informative for allele-assignment
                """,
            plot = bargraph.plot(plot_data, cats, pconfig)
        )

    def allele_sorting_section(self):
        ''' Allele-specific sorting report '''
        cats = OrderedDict()
        cats['sorting_genome1'] = {'name': 'Genome 1'}
        cats['sorting_genome2'] = {'name': 'Genome 2'}
        cats['sorting_unassignable'] = {'name': 'Not assigned'}
        cats['sorting_conflicting'] = {'name': 'Conflicting SNPs'}
        # HiC only
        cats['sorting_genome1_G1_G1'] = {'name': 'Genome 1 / Genome 1'}
        cats['sorting_genome2_G2_G2'] = {'name': 'Genome 2 / Genome 2'}
        cats['sorting_unassignable_UA_UA'] = {'name': 'Unassignable / Unassignable'}
        cats['sorting_G1_UA_total'] = {'name': 'Genome 1 / unassignable'}
        cats['sorting_G2_UA_total'] = {'name': 'Genome 2 / unassignable'}
        cats['sorting_G1_G2_total'] = {'name': 'Different genomes'}

        pconfig = {
            'id': 'snpsplit-sorting-plot',
            'title': "SNPsplit: Allele-specific sorting",
            'ylab': "Reads",
            'cpswitch_counts_label': 'Reads'
        }

        self.add_section(
            name = "Allele-specific sorting",
            description = "Per-sample metrics of how reads and pairs of reads were sorted into each genome.",
            helptext = """
                Bargraph categories are:

                * `Genome 1`: Reads assigned to Genome 1
                * `Genome 2`: Reads assigned to Genome 2
                * `Not assigned`: Reads don't overlap a SNP
                * `Conflicting SNPs`: Reads contained allele-specific information for both alleles within the same read

                For HiC data, categories are:

                * `Genome 1 / Genome 1`: Pairs with both reads specific to genome 1
                * `Genome 2 / Genome 2`: Pairs with both reads specific to genome 2
                * `Unassignable / Unassignable`: Pairs with both reads unassignable (not overlapping a SNP)
                * `Genome 1 / unassignable`: One paired-end read assigned to Genome 1, one unassignable (doesn't overlap a SNP)
                * `Genome 2 / unassignable`: One paired-end read assigned to Genome 2, one unassignable (doesn't overlap a SNP)
                * `Different genomes`: Paired-end reads assigned to different genomes
                * `Conflicting SNPs`: Reads contained allele-specific information for both alleles within the same read

                Allele-specific sorting takes both reads of read pairs or Hi-C samples into account.

                Note that metrics here may differ from those in the allele-tagging report.
                This occurs when paired-end reads are used, since 'tagging' only one read in
                a pair as arising from one genome can suffice in both reads being sorted there.

                Similarly, if two reads in a pair are tagged as arising from different genomes
                then the pair becomes unassignable.
            """,
            plot = bargraph.plot(self.snpsplit_data, cats, pconfig)
        )
